/**
****************************************************************************************************
****************************************************************************************************
****************************************************************************************************
@mainpage gluxDemo project
@version 0.1
@author Juraj Vanek
@date 4.9.2009
@note gluxMark2 demo

@file main.cpp
@brief Entry point for application. Creates windows using SDL, then initializes scene, sets render
loop and receives mouse and keyboard events from user.
****************************************************************************************************
****************************************************************************************************
***************************************************************************************************/
#include "demo.h"
using namespace std;

//FPS & time
unsigned cycle = 0;                     //drawn cycles in one second 
unsigned fps;

//demoscene
TDemo demo;     
TScene *s;
char msg[32];
bool wire = false;                    //wireframe rendering flag
int fsaa;

//*****************************************************************************
//Initialize every test
bool InitScene(int resx, int resy)
{ 
  s = new TScene();
  if(!s->PreInit(resx, resy, 0.1, 1000.0, 45.0, fsaa)) 
    return false;
	if(!demo.InitScenes(s))
		return false;
	return s->PostInit();
}

string name;
//*****************************************************************************
//Main drawing function
void Redraw()
{
  demo.RunScenes(fps);
  cycle++;                          //increment draw counter 

  //FPS info
  sprintf(msg,"%d fps",fps);
  s->DrawScreenText(msg,50.0,1.0,1.25);           
/*
	//meminfo
	int meminfo[4];
	glGetIntegerv(GL_TEXTURE_FREE_MEMORY_ATI,meminfo);
	sprintf(msg,"VRAM used: %d MB",1024 - meminfo[0]/1024);
  s->DrawScreenText(msg,10.0,1.0,1.0); 
*/
}

//*****************************************************************************
//Keyboard input
const float INC = 0.5;
bool use_ssao = true;
void KeyInput(SDLKey key)
{
  switch(key)
  {     
    case SDLK_x:         //X - wireframe
        wire = !wire;
        if(wire) glPolygonMode(GL_FRONT_AND_BACK,GL_LINE);
        else glPolygonMode(GL_FRONT_AND_BACK,GL_FILL);
        break;

		case SDLK_SPACE:
				demo.NextScene();
				break;

    case SDLK_ESCAPE:        //ESCAPE - quit
        SDL_Quit();
        exit(0);
        break;
    default:
        break;
  }
}

/**
****************************************************************************************************
@brief Handles mouse clicks
@param button pressed mouse button
@param state pouse button state
@param x X-coordinate of click
@param y Y-coordinate of click
****************************************************************************************************/
int status;                                     //which button?
void MouseClick(SDL_Event event)
{ 
  if(event.button.state = SDL_RELEASED) 
    status = 0;
  else
    status = event.button.button;
}

/**
****************************************************************************************************
@brief Handles mouse motion
@param x X-coordinate of mouse
@param y Y-coordinate of mouse
****************************************************************************************************/
void MouseMotion(SDL_Event event)
{
  int x = event.motion.xrel;
  int y = event.motion.yrel;

  if(status == SDL_BUTTON_LEFT)   //rotate
  {
    s->RotateCamera(y, A_X);
    s->RotateCamera(x, A_Y);
  }
  else if(status == /* wtf??? proc nefuguje??SDL_BUTTON_RIGHT */ 4)  //zoom
    s->MoveCamera(0.0,0.0, x/10.0 );
  else if(status == SDL_BUTTON_MIDDLE)  //position
    s->MoveCamera( x/10.0, -y/10.0, 0.0);
  
  //print out camera position
  //s->PrintCamera();
}


/**
****************************************************************************************************
@brief Measure time in separate thread - solves problem when drawing is slower than 100fps
@param unused null parameter
****************************************************************************************************/
int Timer(void *unused)
{
  int msec = 0;

	while(true)
	{
		//measure time in miliseconds
		demo.UpdateTime();
		msec++;

		//increment time after one second
    if(msec == 100)
    {
      //save FPS
      fps = cycle;
      cycle = 0;
			msec = 0;
    }
		SDL_Delay(10);
	}
	return 0;
}

/**
****************************************************************************************************
@brief Print program usage if wrong parameters have been used. Then exits program
****************************************************************************************************/
void WrongParams()
{
  cout<<"Wrong parameters.\n"
    "Usage: gluxEngine.exe [-w|-f resX resY][-aa value]\n"
    "Parameters:\n"
    "-w,-f: windowed/fullscreen mode\n"
    "resX, resY: screen resolution in pixels\n"
    "-aa: antialiasing strength (0,1 = off)\n";
  exit(1);
}


///////////////////////////////////////////////////////////////////////////////////////////////////
//********************************* MAIN ********************************************************//
///////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char **argv)
{
  //application settings
  string param;
  fsaa = 1;
  bool fullscreen = false;
  int resx = 1024, resy = 768;

  //parse parameters
  for(int i=1; i<argc; i++)
  {
    param = argv[i];
    ////////////////////////////////////////////////
    //fullscreen/windowed mode
    if(param == "-w" || param == "-f")
    {
      if(param == "-w")
        fullscreen = false;
      else if(param == "-f")
        fullscreen = true;
      //display resolution
      if(i+2 < argc)
      {
        resx = atoi(argv[i+1]);
        resy = atoi(argv[i+2]);
        i += 2;
      }
      else
        WrongParams();
    }
    ///////////////////////////////////////////////
    //antialiasing
    else if(param == "-aa")
    {
      if(i+1 < argc)
      {
        fsaa = atoi(argv[i+1]);
        i++;
      }
      else
        WrongParams();
    }
    ///////////////////////////////////////////
    //error
    else
      WrongParams();
  }

  //initialize SDL video
  if(SDL_Init(SDL_INIT_VIDEO) < 0) exit(1);

  //enable double buffering with a 24bit Z buffer.
  SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER, 1);
  SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);

  //enable multisampling
  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLEBUFFERS, 1);
  SDL_GL_SetAttribute(SDL_GL_MULTISAMPLESAMPLES, fsaa);

  SDL_WM_SetCaption("gluxDemo", 0);

  //set video mode
  int mode = SDL_OPENGL;
  if(fullscreen)
    mode |= SDL_FULLSCREEN;
  if(SDL_SetVideoMode(resx, resy, 32, mode) == NULL)
  {
    ShowMessage("Display mode not supported!\n");
    SDL_Quit();
    return 1;
  }

	//init GLEW
	if(glewInit() != GLEW_OK)
	{
    ShowMessage("GLEW initialization failed!\n");
    SDL_Quit();
    return 1;
	}

  //init scene
  if(!InitScene(resx,resy)) exit(1);

	//measure time in separate thread
	SDL_Thread *time_thread;
  time_thread = SDL_CreateThread(Timer, NULL);

  if(time_thread == NULL )
	{
    ShowMessage("Cannot create timer thread!\n");
#ifdef _WIN_
    ///@todo SDL_Quit crashes on Linux
    SDL_Quit();
#endif
    return 1;
  }

  //Main loop
	SDLKey key;
	bool keypress = false;
  SDL_Event event;
  float last_keypress;
  while(true)
  {
    //handle events with SDL
    if(SDL_PollEvent(&event))
    {
      if(event.type == SDL_QUIT)
        break;
      //keyboard events
      else if(event.type == SDL_KEYDOWN)
				KeyInput(event.key.keysym.sym);
      //mouse events
      else if(event.type == SDL_MOUSEMOTION)
      {
        MouseMotion(event);
        MouseClick(event);
      }
    }

    //call drawing function
    Redraw();
    SDL_GL_SwapBuffers();
  }

  //deinitialize SDL
  SDL_Quit();
  return 0;
}
