/**
****************************************************************************************************
****************************************************************************************************
@file scenes.h
@brief initialization and running of demo
****************************************************************************************************
***************************************************************************************************/
#ifndef _DEMOSCENES_H_
#define _DEMOSCENES_H_
#include "glux_engine/engine.h"


//*********
//ALL SCENES
enum Scenes
{
	INTRO,
  //fillrate
  FILLRATE,
  //fragment shaders
  FRAG_LIGHTS,
  FRAG_PROCEDURAL,
  FRAG_PARALLAX,
  FRAG_REFRACT,
  //geometry shaders
  GEOM_CUBEMAP,
  GEOM_PARTICLES,
	//vertex shaders
	VERT_LIGHTS,
	VERT_WAVES,
	VERT_DISPLACE,
	//complex
	COMPLEX_HDR,
	COMPLEX_SSAO,
	COMPLEX_SHADOW,
	//OpenCL compute
	COMPUTE_NBODY,
	OUTRO
};

///number of fillrate surfaces
const int surfaces = 8;
///number of interacting particles
const int BODIES = 2048;

/**
@brief OpenCL simulation class
**************************************************************************************************/
class NBodySim : public TCompute
{
	private:
		//N-Body simulation
		cl_uint numBodies;
    cl_float espSqr;                    //softening factor

		cl_float* initPos;                  //initial position
		cl_float* pos;											//current position

		cl_float* initVel;                  //initial velocity
		cl_float* vel;                      //current velocity

    cl_mem updatedPos;                  //buffer: position of particles
    cl_mem updatedVel;                  //buffer: velocity of particles

		//performance info
		cl_ulong start, end;
		float kernel_time, flops, total_flops;

	//overrided functions
	public:
		bool Init();
		bool SetupKernelArgs();
		bool Run();
		bool Destroy();

		float GetPos(int index)
		{ return pos[index]; }
		float GetFLOPS()
		{ return flops; }
		float GetTotalFLOPS()
		{ return total_flops; }
};

/**
@brief Demo class
**************************************************************************************************/
class TDemo
{
	private:
		//scene
		TScene *s;
		int current_scene;
		float global_time, local_time, anim;
		string obj_name;
		float exposure;
		string drawtext;
		int object_count;
		vector<float> speeds;

		//lights
		TVector lpos1,lpos2,lpos3,lpos4,lpos5,lpos6,lpos7,lpos8;

		//keyframe animation
		TVector key1_pos, key1_rot,key2_pos, key2_rot, dp, dr;
		int inv;

		//OpenCL compute
		NBodySim *nbody;
		string name;

		int body,halfbody;
		char objname[32];

	public:
		bool InitScenes(TScene *ps);
		void RunScenes(int fps);
		void NextScene();
		void UpdateTime()
		{
			local_time += 0.01;
			global_time += 0.01;
		}
};



#endif
