/**
****************************************************************************************************
****************************************************************************************************
@file scenes.h
@brief initialization and running of demo
****************************************************************************************************
***************************************************************************************************/
#include "demo.h"

/**
**************************************************************************************************
@brief Initialize all scenes
@param s pointer to scene
**************************************************************************************************/
bool TDemo::InitScenes(TScene *ps)
{
	s = ps;
	//basic time settings
	int sID = 0;		//scene ID
	current_scene = INTRO;
	global_time = local_time = 0.0;
	exposure = 1.0;

	//loading polygon
	s->AddMaterial("mat_load_polygon",white,white,white,64.0,0.0,0.0,NONE);
	s->AddTexture("mat_load_polygon","data/load.tga",BASE);
	s->CustomShader("mat_load_polygon","data/shaders/quad.vert",NULL,"data/shaders/load_quad.frag");
	//background
	s->AddMaterial("mat_background_quad",white,white,white,64.0,0.0,0.0,NONE);
	s->AddTexture("mat_background_quad","data/background.tga",BASE);
	s->CustomShader("mat_background_quad","data/shaders/quad.vert",NULL,"data/shaders/load_quad.frag");

	//Initialize everything
	s->SetLoadList(400);

	//cubemap files
	const char *cubemap1[] = { "data/tex/cubemaps/posx.tga", "data/tex/cubemaps/negx.tga",
														 "data/tex/cubemaps/posy.tga", "data/tex/cubemaps/negy.tga",
														 "data/tex/cubemaps/posz.tga", "data/tex/cubemaps/negz.tga" };
	const char *cubemap2[] = { "data/tex/cubemaps/sky/posx.tga", "data/tex/cubemaps/sky/negx.tga", "data/tex/cubemaps/sky/posy.tga",
														 "data/tex/cubemaps/sky/negy.tga", "data/tex/cubemaps/sky/posz.tga", "data/tex/cubemaps/sky/negz.tga" };

	try
	{
	//0) INTRO
			s->ChangeSceneID(sID++);

			s->LoadScene("data/obj/scenes/intro.3ds");
			s->SetShadow(0);
			//additional objects
			s->AddObject("introsky",SPHERE,500.0,0.0,8,8);
			s->AddMaterial("mat_introsky",white,white,white,0.0,0.0,0.0,NONE);
			s->AddTexture("mat_introsky","data/tex/stars.tga",BASE,MODULATE,1.0,5.0,5.0);
			s->CastShadow("introsky",false);
			s->ReceiveShadow("mat_introsky",false);
			s->SetMaterial("introsky","mat_introsky");

	//1) fillrate tests
			s->ChangeSceneID(sID++);
	
			//setup texture surfaces
			s->AddMaterial("fill_mat1",white,white,white,64.0f,0.0f,0.1f,NONE);
			s->AddMaterial("fill_mat2",white,white,white,64.0f,0.0f,0.1f,NONE);

			s->AddTexture("fill_mat1","data/tex/single.tga",BASE,MODULATE,1.0,1.0,1.0);
			s->AddTexture("fill_mat2","data/tex/multi.tga",BASE,MODULATE,1.0,1.0,1.0);

			s->AddObject("fillsky",SPHERE,500.0,0.0,8,8);
			s->RotateObj("fillsky",90.0,A_X);
			s->AddMaterial("mat_fillsky",white,white,white,0.0,0.0,0.0,NONE);
			s->AddTexture("mat_fillsky","data/tex/stars.tga",BASE,MODULATE,1.0,5.0,5.0);
			s->SetMaterial("fillsky","mat_fillsky");
			s->CastShadow("fillsky",false);
			s->ReceiveShadow("mat_fillsky",false);

			//surfaces
			for(int i = 0; i < surfaces; i++)
			{
				obj_name = "surface1" + num2str(i);
				s->AddObject(obj_name.c_str(),PLANE,1.0,1.0);
				s->MoveObjAbs(obj_name.c_str(),-0.65f, -surfaces*0.01f + 0.01f*i,0.0f);
				s->SetMaterial(obj_name.c_str(),"fill_mat1");

				obj_name = "surface2" + num2str(i);
				s->AddObject(obj_name.c_str(),PLANE,1.0,1.0);
				s->MoveObjAbs(obj_name.c_str(),0.65f, -surfaces*0.01f + 0.01f*i,0.0f);
				s->SetMaterial(obj_name.c_str(),"fill_mat2");
			}


	//2) per-pixel lighting
			s->ChangeSceneID(sID++);
	
			lpos1.Set(0.0,10.0,0.0); lpos2.Set(0.0,10.0,0.0); lpos3.Set(0.0,10.0,0.0); lpos4.Set(0.0,10.0,0.0);
			lpos5.Set(0.0,10.0,0.0); lpos6.Set(0.0,10.0,0.0); lpos7.Set(0.0,10.0,0.0); lpos8.Set(0.0,10.0,0.0);

			//add objects
			s->AddObject("room_bottom",PLANE,50.0,100.0); s->AddObject("room_top",PLANE,50.0,100.0);
			s->AddObject("room_left",PLANE,100.0,30.0);   s->AddObject("room_right",PLANE,100.0,30.0);
			s->AddObject("room_front",PLANE,50.0,30.0);   s->AddObject("room_back",PLANE,50.0,30.0);
			s->AddObject("frag_cube1",CUBE,3.0);
			s->AddObject("frag_cube2",CUBE,6.0);
			s->AddObject("frag_sphere","data/obj/teapot.3ds");
			s->ResizeObj("frag_sphere",0.7f,0.7f,0.7f);
			s->MoveObj("frag_sphere",0.0,-1.5,0.0);

			//transform objects
			s->MoveObj("room_top",0.0,30.0,0.0);     s->MoveObj("room_left",-25.0,15.0,0.0);
			s->MoveObj("room_right",25.0,15.0,0.0);  s->MoveObj("room_front",0.0,15.0,50.0);
			s->MoveObj("room_back",0.0,15.0,-50.0);

			s->MoveObj("frag_sphere",0.0,9.0,0.0); s->MoveObj("frag_cube1",0.0,2.5,0.0); s->MoveObj("frag_cube2",0.0,-3.5,0.0);

			s->RotateObj("room_top",  180.0,A_Z);
			s->RotateObj("room_left",  90.0,A_X);
			s->RotateObj("room_left",  -90.0,A_Z);
			s->RotateObj("room_right", 90.0,A_X);
			s->RotateObj("room_right", 90.0,A_Z);
			s->RotateObj("room_front",-90.0,A_X);
			s->RotateObj("room_back",  90.0,A_X);

			//add materials & textures
			s->AddMaterial("mat_frag_floor",black,white,white);
			s->AddMaterial("mat_frag_wall",black,white,white);
			s->AddMaterial("mat_frag_sphere",black,white,white,64.0);

			s->AddTexture("mat_frag_floor","data/tex/wall.tga",BASE,MODULATE,1.0,2.0,2.0,true,true);
			s->AddTexture("mat_frag_floor","data/tex/normal/wall.tga",BUMP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_frag_wall","data/tex/brick.tga",BASE,MODULATE,1.0,3.0,3.0,true,true);
			s->AddTexture("mat_frag_wall","data/tex/normal/brick.tga",BUMP,MODULATE,1.0,3.0,3.0);
			s->AddTexture("mat_frag_sphere", "data/tex/normal/bump.tga",BUMP,MODULATE,0.2f, 1.0f,1.0f);

			//set materials
			s->SetMaterial("room_left","mat_frag_wall");  s->SetMaterial("room_right","mat_frag_wall");
			s->SetMaterial("room_front","mat_frag_wall"); s->SetMaterial("room_back","mat_frag_wall");
			s->SetMaterial("room_top","mat_frag_floor");  s->SetMaterial("room_bottom","mat_frag_floor");
			s->SetMaterial("frag_sphere","mat_frag_sphere");
			s->SetMaterial("frag_cube1","mat_frag_floor");
			s->SetMaterial("frag_cube2","mat_frag_floor");


	//3) procedural shaders
			s->ChangeSceneID(sID++);

			s->AddObject("proc_teapot","data/obj/teapot.3ds");
			s->ResizeObj("proc_teapot",0.15f,0.15f,0.15f);
			s->MoveObj("proc_teapot",0.0,-1.0,0.0);
			s->AddObject("proc_cube",CUBE,1.0);
			s->AddObject("proc_sphere",SPHERE,1.0,1.0,32,32);
			s->AddObject("proc_sky",SPHERE,20.0,20.0,32,32);
			s->AddObject("proc_cylinder","data/obj/cylinder.3ds");
			s->AddObject("proc_torus",TORUS,0.5,1.25,8,32);
			s->AddObject("proc_cone","data/obj/cone.3ds");

			s->MoveObj("proc_cube",-4.0,0.0,0.0);
			s->MoveObj("proc_sphere",4.0,0.0,0.0);
			s->MoveObj("proc_cylinder",0.0,-4.0,0.0);
			s->MoveObj("proc_torus",-4.0,-4.0,0.0);
			s->MoveObj("proc_cone", 4.0,-4.0,0.0);

			s->AddMaterial("mat_proc_wood",silver,white,white,16);
			s->CustomShader("mat_proc_wood","data/shaders/procedural/wood.vert",NULL,"data/shaders/procedural/wood.frag");
			s->SetMaterial("proc_cube","mat_proc_wood");

			s->AddMaterial("mat_proc_marble",black,white,white,128);
			s->CustomShader("mat_proc_marble","data/shaders/procedural/marble.vert",NULL,"data/shaders/procedural/marble.frag");
			s->SetMaterial("proc_cone","mat_proc_marble");

			s->AddMaterial("mat_proc_bump",black,yellow,red,64);
			s->CustomShader("mat_proc_bump","data/shaders/procedural/bump.vert",NULL,"data/shaders/procedural/bump.frag");
			s->SetMaterial("proc_torus","mat_proc_bump");

			s->AddMaterial("mat_proc_lattice",black,white,white,128);
			s->CustomShader("mat_proc_lattice","data/shaders/procedural/lattice.vert",NULL,"data/shaders/procedural/lattice.frag");
			s->SetMaterial("proc_sphere","mat_proc_lattice");

			s->AddMaterial("mat_proc_brick",black,white,white,128);
			s->CustomShader("mat_proc_brick","data/shaders/procedural/brick.vert",NULL,"data/shaders/procedural/brick.frag");
			s->SetMaterial("proc_cylinder","mat_proc_brick");

			s->AddMaterial("mat_proc_phong",black,white,white,64);
			s->CustomShader("mat_proc_phong","data/shaders/procedural/phong.vert",NULL,"data/shaders/procedural/phong.frag");
			s->SetMaterial("proc_teapot","mat_proc_phong");

			s->AddMaterial("mat_proc_sky");
			s->CustomShader("mat_proc_sky","data/shaders/procedural/noise.vert",NULL,"data/shaders/procedural/noise.frag");
			s->SetMaterial("proc_sky","mat_proc_sky");

			s->SetUniform("mat_proc_sky","clamp_top",0.8);
			s->SetUniform("mat_proc_sky","clamp_bottom",0.2);
			s->SetUniform("mat_proc_sky","octaves",4.0);


	//4) parallax mapping
			s->ChangeSceneID(sID++);
	
			s->AddObject("plx_obj",PLANE,17.0,9.0);
			s->RotateObj("plx_obj",90.0,A_X);

			s->AddMaterial("mat_parallax");
			s->AddTexture("mat_parallax","data/tex/wall.tga",BASE,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/normal/wall.tga",BUMP,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/normal/tree.tga",BUMP,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/depth/wall.tga",PARALLAX,MODULATE,0.0015f,1.0f,1.0f);
			s->AddTexture("mat_parallax","data/tex/depth/tree.tga",PARALLAX,MODULATE,0.0015f,1.0f,1.0f);
			s->SetMaterial("plx_obj","mat_parallax");


	//5) refraction
			s->ChangeSceneID(sID++);

			s->AddObject("refr_obj1",SPHERE,3.0,4.0,48,48);
			s->AddObject("refr_obj2",TORUS,1.0,4.0,32,48);
			s->AddObject("refr_obj3",TORUS,1.0,4.0,32,48);
			s->RotateObj("refr_obj3",90.0f,A_Y);
			s->AddObject("refr_sky",SPHERE,200.0,0.0,32,32);

			s->AddMaterial("mat_ref_chroma");
			s->AddMaterial("mat_refr_sky",black,white,white,32.0,0.0,0.0,NONE);

			s->AddTexture("mat_refr_sky",cubemap1,CUBEMAP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_ref_chroma",cubemap1,CUBEMAP,MODULATE,1.0,2.0,2.0);

			s->SetMaterial("refr_sky","mat_refr_sky");
			s->SetMaterial("refr_obj1","mat_ref_chroma");
			s->SetMaterial("refr_obj2","mat_ref_chroma");
			s->SetMaterial("refr_obj3","mat_ref_chroma");
			s->CustomShader("mat_ref_chroma","data/shaders/refract.vert",NULL,"data/shaders/refract.frag",NULL,"#define USE_BUILTIN\n#define CHROMA\n");

	//6) reflexion
			s->ChangeSceneID(sID++);

			s->AddObject("rflx_sphere",SPHERE,1.0,1.0,32,32);
			s->AddObject("rflx_sphere2",SPHERE,0.5,0.5,32,32);
			s->AddObject("rflx_sky",SPHERE,100.0,0.0,32,32);

			s->AddMaterial("mat_rflx_sphere",dgrey,white,white,32.0,1.0,0.0, GOURAUD);
			s->AddMaterial("mat_rflx_sphere2",dgrey,dgrey,white,32.0,1.0,0.0, GOURAUD);
			s->AddMaterial("mat_rflx_fighter",black,white,white,64.0, 0.0, 0.0, GOURAUD);
			s->AddMaterial("mat_rflx_sky",white,white,white,1.0,0.0,0.0,NONE);

			//add textures
			s->AddTexture("mat_rflx_sky", "data/tex/stars.tga",BASE, MODULATE, 1.0, 4.0,4.0);
			s->AddTexture("mat_rflx_fighter", "data/tex/fighter.tga",BASE);
			s->AddTexture("mat_rflx_sphere", "data/tex/metal.tga",BASE, MODULATE, 0.5, 2.0, 2.0);
			s->AddTexture("mat_rflx_sphere2", "data/tex/metal.tga",BASE, MODULATE, 0.5, 2.0, 2.0);

			//setup materials
			s->SetMaterial("rflx_sphere", "mat_rflx_sphere");
			s->SetMaterial("rflx_sphere2", "mat_rflx_sphere2");
			s->SetMaterial("rflx_sky", "mat_rflx_sky");

			//many planes
			float rnd1, rnd2, rnd3;
			s->AddObject("fighter0","data/obj/fighter.3ds");
			s->SetMaterial("fighter0", "mat_rflx_fighter");
			object_count = 7;
			for(int i=0; i<object_count; i++)
			{
				for(int j=0; j<object_count; j++)
				{
					rnd1 = rand()%100 / 100.0f - 2.0f;
					rnd2 = rand()%100 / 100.0f - 2.0f;
					rnd3 = rand()%10 / 10.0f;
					obj_name = "fighter" + num2str(i*object_count + j + 1);
					s->AddObjectInstance("fighter0",obj_name.c_str());
					s->MoveObj(obj_name.c_str(), 40.0f, (2.0f * i + rnd1) - (2.0f * object_count/2), (5.0f * j + rnd2) - (5.0f * object_count/2) );
					speeds.push_back(rnd3);
				}
			}
			s->SetReflectionSize("rflx_sphere",256);
			s->SetReflectionSize("rflx_sphere2",256);
			s->UseDynReflections(false);      //create dynamic cubemaps

	//7) particles
			s->ChangeSceneID(sID++);

			//wet plane
			s->AddObject("wet_plane",PLANE,50.0,50.0);

			s->AddMaterial("mat_rain",black,dgrey,white,64.0,0.0,1.0);
			s->AddTexture("mat_rain","data/tex/alpha/raindrop.tga");
			s->AddMaterial("mat_wet",black,dgrey,white,8.0);
			s->AddTexture("mat_wet","data/tex/wall.tga");
			s->AddTexture("mat_wet","data/tex/normal/bump.tga",BUMP);
			s->AddTexture("mat_wet","data/tex/normal/raindrops.tga",BUMP);
			s->SetMaterial("wet_plane","mat_wet");

			//position rain emitters
			float rnd;
			object_count = 20;
			s->AddObject("emitter0",PLANE,10.0,10.0);
			s->SetMaterial("emitter0", "mat_rain");
			for(int i=0; i<object_count; i++)
			{
				for(int j=0; j<object_count; j++)
				{
					rnd = (rand()%10)/5.0f - 1.0f;
					obj_name = "emitter" + num2str(i*object_count + j + 1);
					s->AddObjectInstance("emitter0",obj_name.c_str());
					s->MoveObj(obj_name.c_str(), 1.5f*j + rnd - object_count/2.0f, 10.0f - 10.0f*rnd, 1.5f*i - object_count/2.0f);
				}
			}
			//rain shader
			s->CustomShader("mat_rain","data/shaders/geometry/rain.vert","data/shaders/geometry/rain.geom","data/shaders/geometry/rain.frag",NULL,NULL,
				"#define PARTICLES 4\nlayout(max_vertices=64) out;");
			s->CustomShader("mat_wet","data/shaders/wet.vert",NULL,"data/shaders/wet.frag");

			//starting setup
			s->SetUniform("mat_wet","rain_speed",30.0);

	//8) per-vertex lighting & GI
			s->ChangeSceneID(sID++);

			int INSTANCES = 10;
			//car
			s->LoadScene("data/obj/scenes/bmw.3ds");
			s->DrawObject("glass",false);
			s->AddMaterial("mat_gi_body",black,silver,white,64.0,0.0,0.0,GOURAUD);
			s->AddTexture("mat_gi_body",cubemap1,CUBEMAP_ENV);
			s->SetMaterial("body","mat_gi_body");

			//ground
			s->AddMaterial("mat_gi_tile",black,white,white,8.0,0.0,0.0,GOURAUD);
			s->AddTexture("mat_gi_tile","data/tex/metal.tga");

			s->AddObject("tile0","data/obj/tile.3ds");
			s->MoveObj("tile0", -INSTANCES*2.0f, -1.7f, -INSTANCES*2.0f);
			s->SetMaterial("tile0", "mat_gi_tile");
			//geometry instanced floor (if supported)
      if(GLEW_ARB_draw_instanced || GLEW_EXT_draw_instanced)
      {
			  s->CustomShader("mat_gi_tile","data/shaders/instanced.vert",NULL,"data/shaders/instanced.frag");
			  s->SetUniform("mat_gi_tile","INSTANCES",INSTANCES);
			  s->SetGInstances("tile0",INSTANCES*INSTANCES);
      }
      //else use less effective method of instancing
      else
      {
        string obj_name;
        for(int i=0; i<INSTANCES*INSTANCES; i++)
			  {
				  obj_name = "tile0" + num2str(i + 1);
				  s->AddObjectInstance("tile0",obj_name.c_str());
				  s->MoveObj(obj_name.c_str(), 5.0f*(i%INSTANCES), 0.0f, 5.0f*(i/INSTANCES));
			  }
      }

	//9) gpu waves
			s->ChangeSceneID(sID++);

			s->AddObject("wave_sky",SPHERE,200.0,0.0,8,8);
			s->AddObject("wave_sea",STRIP_PLANE,500.0,500.0,500,500);

			//add materials & textures
			s->AddMaterial("mat_sea",silver,white,white,16.0);
			s->AddMaterial("mat_wave_sky",black,white,white,0.0,0.0,0.0,NONE);

			s->AddTexture("mat_wave_sky",cubemap2);
			s->AddTexture("mat_sea",cubemap2,CUBEMAP_ENV);
			s->AddTexture("mat_sea","data/tex/sea.tga");

			s->CustomShader("mat_sea","data/shaders/sea.vert",NULL,"data/shaders/sea.frag");

			//set materials
			s->SetMaterial("wave_sky","mat_wave_sky");
			s->SetMaterial("wave_sea","mat_sea");

	//10) displacement mapping
			s->ChangeSceneID(sID++);

			s->AddObject("disp_tree_top","data/obj/tree_top.3ds");
			s->AddObject("disp_tree_bark","data/obj/tree_bark.3ds");
			s->AddObject("disp_terrain",STRIP_PLANE,300.0,300.0,200,200);
			s->MoveObj("disp_terrain",-30.0,-0.5,0.0);
			s->AddObject("disp_sky",SPHERE,200.0,0.0,8,8);
			s->AddObject("disp_ground",STRIP_PLANE,50.0,100.0,500,1000);

			//add lights
			s->AddLight(0,grey,white,white,TVector(25.0,65.0,125.0),10000.0);

			//add materials & textures
			s->AddMaterial("disp_mat_tree_top",black,white,white,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("disp_mat_tree_bark",black,white,white,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_ground",black,white,silver,128.0,0.0,0.0,GOURAUD);
			s->AddMaterial("disp_mat_terrain",black,white,grey,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("disp_mat_sky",black,white,white,0.0,0.0,0.0,NONE);
			s->AddTexture("disp_mat_sky",cubemap2);

			//displacement material textures
			s->AddTexture("disp_mat_tree_top","data/tex/tree.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("disp_mat_tree_top","data/tex/normal/tree.tga",BUMP, MODULATE, 1.0, 2.0, 4.0);
			s->AddTexture("disp_mat_tree_top","data/tex/depth/tree.tga",DISPLACE,MODULATE,0.1f, 1.0f, 1.0f);

			s->AddTexture("disp_mat_tree_bark","data/tex/tree.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("disp_mat_tree_bark","data/tex/normal/tree.tga",BUMP, MODULATE, 1.0, 2.0, 4.0);
			s->AddTexture("disp_mat_tree_bark","data/tex/depth/tree.tga",DISPLACE,MODULATE,0.3f, 0.25f, 0.25f);

			s->AddTexture("mat_ground","data/tex/dry.tga",BASE,MODULATE,1.0,2.0,4.0);
			s->AddTexture("mat_ground","data/tex/normal/dry.tga",BUMP, MODULATE, 1.0, 2.0,4.0);
			s->AddTexture("mat_ground","data/tex/depth/dry.tga",DISPLACE,MODULATE, 2.0,2.0,4.0);
			s->AddTexture("mat_ground","data/tex/alpha/tree_shadow.tga",BASE,MODULATE);

			s->AddTexture("disp_mat_terrain","data/tex/dust.tga",BASE,MODULATE,1.0,20.0,20.0);
			s->AddTexture("disp_mat_terrain","data/tex/normal/terrain.tga",BUMP);
			s->AddTexture("disp_mat_terrain","data/tex/depth/terrain.tga",DISPLACE,MODULATE, 0.0);

			//modified shader with static shadow map
			s->CustomShader("mat_ground","data/shaders/displaced_ground.vert",NULL,"data/shaders/displaced_ground.frag");

			//set materials
			s->SetMaterial("disp_sky","disp_mat_sky");
			s->SetMaterial("disp_tree_top","disp_mat_tree_top");
			s->SetMaterial("disp_tree_bark","disp_mat_tree_bark");
			s->SetMaterial("disp_ground","mat_ground");
			s->SetMaterial("disp_terrain","disp_mat_terrain");

	//11) HDR
			s->ChangeSceneID(sID++);

			//load scenes
			s->LoadScene("data/obj/scenes/bmw.3ds",true,true,"h_");
			s->LoadScene("data/obj/scenes/hills.3ds");
			//additional materials/textures
			s->AddTexture("hmatroad","data/tex/road.tga",BASE, MODULATE, 1.0, 1.0, 1.0, true, true);
			s->AddTexture("hmatroad","data/tex/normal/bump.tga",BUMP, MODULATE, 1.0, 2.0, 2.0, true, true);
			s->AddTexture("hmatgrass","data/tex/grass.tga",BASE, MODULATE, 0.5, 10.0, 20.0, true, true);
			s->AddTexture("hmathills","data/tex/dirtgras.tga",BASE, MODULATE, 0.5, 10.0, 20.0);
			s->AddMaterial("hmat_body",silver);
			s->AddTexture("hmat_body",cubemap1,CUBEMAP_ENV);
			s->SetMaterial("h_body","hmat_body");
			s->AddMaterial("hmat_glass",black, white, white, 64.0, 0.0, 1.0);
			s->AddTexture("hmat_glass",cubemap1,CUBEMAP_ENV);
			s->SetMaterial("h_glass","hmat_glass");
			s->AddMaterial("hmat_chrome",black, white, white);
			s->AddTexture("hmat_chrome",cubemap1,CUBEMAP_ENV,ADD);
			s->SetMaterial("h_metal","hmat_chrome");

			//load objects
			s->AddObject("hdr_skybox",SPHERE,400.0,0.0,8,8);
			s->MoveObj("hdr_skybox",0.0, 100.0, 0.0);

			//add materials
			s->AddMaterial("hdr_mat_sky",white,white,white,0.0,0.0,0.0,NONE);
			s->AddMaterial("hdr_mat_bark",lgrey,white);
			s->AddMaterial("hdr_mat_leaf",lgrey,white);
			s->AddMaterial("hdr_mat_grass",green,white);

			//add textures
			s->AddTexture("hdr_mat_bark","data/tex/bark4.tga");
			s->AddTexture("hdr_mat_bark","data/tex/normal/bark4_nm.tga",BUMP,MODULATE, 1.0);
			s->AddTexture("hdr_mat_leaf","data/tex/alpha/leafs4.tga",ALPHA);
			s->AddTexture("hdr_mat_sky",cubemap1);
			s->AddTexture("hdr_mat_grass","data/tex/alpha/grass.tga",ALPHA);

			//set materials
			s->SetMaterial("hdr_skybox","hdr_mat_sky");

			//trees&grass
			int OBJECT_COUNT_X = 7;
			int OBJECT_COUNT_Y = 4;
			string bark, leaf;
			for(int i=0; i<OBJECT_COUNT_X; i++)
			{
				for(int j=0; j<OBJECT_COUNT_Y; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					bark = "hdr_tree" + num2str(i*OBJECT_COUNT_X + j) + "_bark";
					leaf = "hdr_tree" + num2str(i*OBJECT_COUNT_Y + j) + "_leaf";
					//two tree types
					if( (i*OBJECT_COUNT_X + j)%2 == 1)
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree1_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree1_leaf.3ds");
					}
					else
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree2_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree2_leaf.3ds");
					}
					float movz = i*14.0f + rnd - 7.0f*OBJECT_COUNT_X + 10.0f;
					float movx = j*14.0f - rnd - 7.0f*OBJECT_COUNT_Y - 30.0f;
					float roty = 10.0f * rnd;
					s->MoveObj(bark.c_str(), movx, 2.0f-j, movz);
					s->MoveObj(leaf.c_str(), movx, 2.0f-j, movz);
					s->RotateObj(bark.c_str(), roty, A_Y);
					s->RotateObj(leaf.c_str(), roty, A_Y);
					s->SetMaterial(bark.c_str(), "hdr_mat_bark");
					s->SetMaterial(leaf.c_str(), "hdr_mat_leaf");
				}
			}

			//shadow settings
			s->CastShadow("h_glass",false);
			s->CastShadow("hcesta",false);
			s->CastShadow("hhory",false);
			s->CastShadow("hdr_skybox",false);
			s->ReceiveShadow("hdr_mat_sky",false);

	//12) SSAO
			s->ChangeSceneID(sID++);
			//load scene
			s->LoadScene("data/obj/scenes/room.3ds");

	//13) Shadows
			s->ChangeSceneID(sID++);

			s->LoadScene("data/obj/scenes/chata.3ds");
			//additional materials&textures
			s->AddMaterial("sh_mat_sky",white,white,white,1.0,0.0,0.0,NONE);
			s->AddTexture("shmatleaf","data/tex/alpha/leafs4.tga",ALPHA);
			s->AddTexture("shmatgrass","data/tex/alpha/grass.tga",ALPHA);
			s->AddTexture("shmatroad","data/tex/dirtroad.tga",BASE,MODULATE,1.0,1.0,1.0,true,true);
			s->AddTexture("shmatroad","data/tex/normal/bump.tga",BUMP,MODULATE,1.0,1.0,1.0,true,true);
			s->AddTexture("shmatroof","data/tex/roof.tga",BASE,MODULATE,1.0,1.0,1.0,true,true);
			s->AddTexture("sh_mat_sky",cubemap2);
			//load objects
			s->AddObject("sh_sky",SPHERE,200.0,0.0,8,8);

			//set materials
			s->SetMaterial("sh_sky","sh_mat_sky");

			//cast/receive shadows
			s->CastShadow("sh_sky",false);
			s->CastShadow("ground",false);
			s->ReceiveShadow("sh_mat_sky",false);

	//14) NBody 
			s->ChangeSceneID(sID++);

			halfbody = BODIES/2;
			nbody = new NBodySim();
			if(!nbody->Init())
				throw ERR;
			if(!nbody->SetupKernel("data/kernels/NBody.cl","nbody_sim"))
				throw ERR;

			s->AddMaterial("cl_mat",black,white,black,0.0,0.0,0.0,NONE);
			s->AddMaterial("cl_mat2",black,TVector(1.0f,0.8f,0.6f),black,0.0,0.0,0.0,NONE);

			//sky(space :)
			s->AddObject("cl_space",SPHERE,500.0,0.0,8,8);
			s->AddMaterial("cl_mat_space",white,white,white,0.0,0.0,0.0,NONE);
			s->AddTexture("cl_mat_space","data/tex/stars.tga",BASE,MODULATE,1.0,6.0,4.0);
			s->SetMaterial("cl_space","cl_mat_space");

			//random position, two groups

			//first group of instances
			s->AddObject("1_star0",SPHERE,0.3f,0.0f,4,4);
			s->SetMaterial("1_star0","cl_mat");
			for(int i=0; i<BODIES/2; i++)
			{
				string name = "1_star" + num2str(i + 1);
				s->AddObjectInstance("1_star0",name.c_str());
			}
			//second group of instances
			s->AddObject("2_star0",SPHERE,0.6f,0.0f,4,4);
			s->SetMaterial("2_star0","cl_mat2");
			for(int i=0; i<BODIES/2; i++)
			{
				string name = "2_star" + num2str(i + 1);
				s->AddObjectInstance("2_star0",name.c_str());
			}

	//15) Outro
			s->ChangeSceneID(sID++);
			//load scenes
			s->LoadScene("data/obj/scenes/forest.3ds");
			s->AddMaterial("mat_well",grey);
			s->AddTexture("mat_well","data/tex/wall.tga",BASE, MODULATE, 1.0, 4.0, 4.0);
			s->AddTexture("mat_well","data/tex/normal/wall.tga",BUMP, MODULATE, 1.0, 4.0, 4.0);
			s->SetMaterial("well","mat_well");

			//load objects
			s->AddObject("out_skybox",SPHERE,400.0,0.0,8,8);
			s->AddObject("out_ground",PLANE,200.0,200.0);

			//add materials
			s->AddMaterial("out_mat_bark",lgrey,white);
			s->AddMaterial("out_mat_leaf",lgrey,white);
			s->AddMaterial("out_mat_grass",lgrey,white);
			s->AddMaterial("out_mat_ground",silver,silver);
			s->AddMaterial("out_mat_forest",white, white);
			s->AddMaterial("out_mat_sky",white,white,white,0.0,0.0,0.0,NONE);

			//add textures
			s->AddTexture("out_mat_bark","data/tex/bark4.tga");
			s->AddTexture("out_mat_bark","data/tex/normal/bark4_nm.tga",BUMP,MODULATE, 1.0);
			s->AddTexture("out_mat_leaf","data/tex/alpha/leafs4.tga",ALPHA);
			s->AddTexture("out_mat_grass","data/tex/alpha/grass.tga",ALPHA);
			s->AddTexture("out_mat_sky",cubemap2);
			s->AddTexture("out_mat_ground","data/tex/grass.tga",BASE, MODULATE, 1.0, 10.0, 20.0);
			s->AddTexture("out_mat_forest","data/tex/alpha/forest.tga",ALPHA,MODULATE, 1.0, 10.0);

			//set materials
			s->SetMaterial("out_skybox","out_mat_sky");
			s->SetMaterial("out_ground","out_mat_ground");
			s->SetMaterial("forest","out_mat_forest");

			//trees
			int OBJECT_COUNT = 7;
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					bark = "out_tree" + num2str(i*OBJECT_COUNT + j) + "_bark";
					leaf = "out_tree" + num2str(i*OBJECT_COUNT + j) + "_leaf";
					//two tree types
					if( (i*OBJECT_COUNT + j)%2 == 1)
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree1_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree1_leaf.3ds");
					}
					else
					{
						s->AddObject(bark.c_str(),"data/obj/trees/tree2_bark.3ds");
						s->AddObject(leaf.c_str(),"data/obj/trees/tree2_leaf.3ds");
					}
					float movx = j*14.0f + rnd - 7.0f*OBJECT_COUNT;
					float movz = i*14.0f - rnd - 7.0f* OBJECT_COUNT;
					float roty = 10.0f * rnd;
					s->MoveObj(bark.c_str(), movx, 0.0, movz);
					s->MoveObj(leaf.c_str(), movx, 0.0, movz);
					s->RotateObj(bark.c_str(), roty, A_Z);
					s->RotateObj(leaf.c_str(), roty, A_Z);
					s->SetMaterial(bark.c_str(), "out_mat_bark");
					s->SetMaterial(leaf.c_str(), "out_mat_leaf");
				}
			}

			//grass - one object and then instances
			OBJECT_COUNT = 20;
			s->AddObject("out_grass","data/obj/trs.3ds");
			s->SetMaterial("out_grass", "out_mat_grass");
			string grass;
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					grass = "out_grass" + num2str(i*OBJECT_COUNT + j);
					s->AddObjectInstance("out_grass",grass.c_str());
					float movx = j*3.0f + rnd - OBJECT_COUNT;
					float movz = i*3.0f - rnd - OBJECT_COUNT;
					float roty = 10.0f * rnd;
					s->MoveObj(grass.c_str(), movx, 0.0, movz);
					s->RotateObj(grass.c_str(), roty, A_Z);
					s->CastShadow(grass.c_str(),false);
				}
			}
			//shadow settings
			s->CastShadow("out_skybox",false);
			s->CastShadow("out_ground",false);
			s->CastShadow("forest",false);
			s->ReceiveShadow("out_mat_sky",false);
			s->ReceiveShadow("out_mat_forest",false);

	}
	catch(int err)
	{
		return false;
	}

	//toggle HDR
	s->UseHDR(true);
	s->UseSSAO(true);

	//prepare render to texture
	s->CreateRenderTarget();
	s->UseSSAO(false);

	//General HDR settings
	s->SetUniform("mat_bloom_hdr","THRESHOLD",0.0);
	s->SetUniform("mat_tonemap","exposure",exposure);
	s->SetUniform("mat_tonemap","bloomFactor",1.0);
	s->SetUniform("mat_tonemap","brightMax",1.5);

	//blur
	s->SetUniform("mat_blur_horiz","kernel_size",11);
	s->SetUniform("mat_blur_horiz","blur_radius",0.003);
	s->SetUniform("mat_blur_vert","kernel_size",11);
	s->SetUniform("mat_blur_vert","blur_radius",0.003);

	//set default sceneID
	s->ChangeSceneID(0);

	//initial camera setting
	s->MoveCameraAbs(17.0,-13,-120);
	s->RotateCameraAbs(10.0,A_X);
	s->RotateCameraAbs(30.0,A_Y);

	//intro music
	PlaySound("data/audio/intro.wav",NULL,SND_ASYNC);
	return true;
}
