/**
****************************************************************************************************
****************************************************************************************************
@file scenes.h
@brief initialization and running of demo
****************************************************************************************************
***************************************************************************************************/
#include "demo.h"


/**
**************************************************************************************************
@brief Jump to next scene
**************************************************************************************************/
void TDemo::NextScene()
{
	current_scene++;
	local_time = 0.0;
	s->ChangeSceneID(current_scene);
	s->RemoveLight();
	//reset camera
	s->MoveCameraAbs(0.0,0.0,0.0);
	s->RotateCameraAbs(0.0,A_X); s->RotateCameraAbs(0.0,A_Y); s->RotateCameraAbs(0.0,A_Z);

	//setup camera and lights
  if(current_scene == FILLRATE)
	{
		s->MoveCamera(0.0f,0.0f,-2.5f);
		s->RotateCamera(90.0,A_X);
	}
	else if(current_scene == FRAG_LIGHTS)
	{
		// setup camera
		s->MoveCamera(0.0,-11.5,-49.0);
		s->RotateCamera(30.0,A_Y);
		//basic light positions
		lpos1.Set(0.0,10.0,0.0); lpos2.Set(0.0,10.0,0.0); lpos3.Set(0.0,10.0,0.0); lpos4.Set(0.0,10.0,0.0);
		lpos5.Set(0.0,10.0,0.0); lpos6.Set(0.0,10.0,0.0); lpos7.Set(0.0,10.0,0.0); lpos8.Set(0.0,10.0,0.0);
		//add 8 lights
		s->AddLight(0,black,white,white,lpos1,40.0);
		s->AddLight(1,black,red,red,lpos2);
		s->AddLight(2,black,green,green,lpos3);
		s->AddLight(3,black,blue,blue,lpos4);
		s->AddLight(4,black,yellow,yellow,lpos5);
		s->AddLight(5,black,magenta,magenta,lpos6);
		s->AddLight(6,black,cyan,cyan,lpos7);
		s->AddLight(7,black,silver,silver,lpos8);
	}
	else if(current_scene == FRAG_PROCEDURAL)
	{
		s->MoveCamera(0.0,2.0,-10.0);
    s->RotateCamera(15.0,A_X);
    s->RotateCamera(-15.0,A_Y);
		s->AddLight(0,black,silver,white,TVector(3.0,3.0,3.0));
	}
	else if(current_scene == FRAG_PARALLAX)
	{
		s->MoveCamera(0.0,0.0,-10.0);
		lpos1.Set(0.0,0.0,12.0);
		s->AddLight(0,black,white,white,lpos1);
	}
	else if(current_scene == FRAG_REFRACT)
	{
		s->MoveCamera(0.0,0.0,-15.0);
    s->RotateCamera(30.0,A_Y);
	}
	else if(current_scene == GEOM_CUBEMAP)
	{
    s->MoveCamera(0.0,0.0,-10.0);
    s->RotateCamera(15.0,A_Y);
    s->RotateCamera(15.0,A_X);
		s->AddLight(0,silver,TVector(1.0f,0.9f,0.8f),white,TVector(2.0,5.0,3.0));
	}
	else if(current_scene == GEOM_PARTICLES)
	{
    s->MoveCamera(-4.0,0.0,-15.0);
    s->RotateCamera(35.0,A_X);
		s->AddLight(0,black,dgrey,white,TVector(0.0,10.0,10.0));
	}
	else if(current_scene == VERT_LIGHTS)
	{
	  //camera position
		s->RotateCamera(-45.0,A_Y);
		s->RotateCamera(10.0,A_X);
	  //basic light positions
	  lpos1.Set(0.0,10.0,0.0); lpos2.Set(0.0,10.0,0.0); lpos3.Set(0.0,10.0,0.0); lpos4.Set(0.0,10.0,0.0);
		//add 4 starting lights
	  s->AddLight(0,black,white,white,lpos1,40.0);
    s->AddLight(1,black,red,red,lpos2);
    s->AddLight(2,black,green,green,lpos3);
    s->AddLight(3,black,blue,blue,lpos4);
	}
	else if(current_scene == VERT_WAVES)
	{
		s->MoveCamera(0.0,-5.0,-5.0);
		s->AddLight(0,grey,white,white,TVector(0.5,20.0,1.5));
	}
	else if(current_scene == VERT_DISPLACE)
	{
		s->MoveCamera(0.0,-8.0,-60.0);
		s->RotateCamera(10.0,A_X);
		s->AddLight(0,grey,white,white,TVector(25.0,65.0,125.0));
	}
	else if(current_scene == COMPLEX_HDR)
	{
		//keyframe animation (linear interpolation)
		key1_pos.Set(3.1f, -8.7f, -12.8f);
		key1_rot.Set(-25.0f, -200.0f, 0.0f);
    key2_pos.Set(2.2f, -2.7f, -22.2f);
		key2_rot.Set( 12.0f, -35.0f, 0.0f);
		inv = 1;

		s->MoveCamera(key1_pos.x, key1_pos.y, key1_pos.z);
		s->RotateCamera(key1_rot.x, A_X);
		s->RotateCamera(key1_rot.y, A_Y);

		s->AddLight(0, silver, white, white, TVector(30.0, 40.0, 30.0));
		s->SetShadow(0,1024);
	}
	else if(current_scene == COMPLEX_SSAO)
	{
		//add lights
		s->AddLight(0, black, silver, white, TVector(130.0,25.0,0.0),150);
		s->AddLight(1, black, silver, white, TVector(-130.0,25.0,0.0),150);
		s->AddLight(2, black, TVector(1.0,0.3,0.0), white, TVector(0.0,60.0,0.0),150);

		//adjust some parameters
		s->MoveCameraAbs(6.5,-15.0,-90.0);
		s->RotateCamera(15.0,A_X);
		s->SetLightRadius(0,150.0);

		//toggle SSAO
		s->UseSSAO(true);

		//SSAO settings
		s->SetUniform("mat_bloom_hdr_ssao","totStrength",2.2);
		s->SetUniform("mat_bloom_hdr_ssao","rad",0.1);
		s->SetUniform("mat_bloom_hdr_ssao","bias",0.02);

		//blur
		s->SetUniform("mat_blur_horiz","kernel_size",15);
		s->SetUniform("mat_blur_horiz","blur_radius",0.0025);
		s->SetUniform("mat_blur_vert","kernel_size",15);
		s->SetUniform("mat_blur_vert","blur_radius",0.0025);
	}
	else if(current_scene == COMPLEX_SHADOW)
	{

		s->UseSSAO(false);
		//position camera
		s->MoveCamera(1.0,-3.0,-25.0);
		s->RotateCamera(5.0, A_X);
		s->RotateCamera(60.0, A_Y);

		//lights
		lpos1.Set(-30.0,45.0,5.0);
		s->AddLight(0,dgrey,white,white,lpos1);
		s->AddLight(1,dgrey,silver,grey,TVector(-20.0,20.0,10.0));
		s->AddLight(2,dgrey,silver,grey,TVector(-20.0,20.0,-10.0));
		s->SetShadow(0,3072);
	}
	else if(current_scene == COMPUTE_NBODY)
	{
		//position camera
		s->MoveCamera(0.0,0.0,-150.0);
		s->RotateCamera(15.0, A_X);

		//General HDR settings
		s->SetUniform("mat_bloom_hdr","THRESHOLD",0.2);
		s->SetUniform("mat_tonemap","exposure",1.5);
		s->SetUniform("mat_tonemap","bloomFactor",1.0);
		s->SetUniform("mat_tonemap","brightMax",1.0);
	}
	else if(current_scene == OUTRO)
	{
		//position camera
		s->MoveCamera(0.0, -4.5, -15.0);
		s->RotateCamera(-8.0, A_X);		
		s->AddLight(0, lgrey, white, white, TVector(30.0, 40.0, 30.0));
		s->SetShadow(0,512);

		//toggle SSAO
		s->UseSSAO(true);

		//SSAO settings
		s->SetUniform("mat_bloom_hdr_ssao","totStrength",3.0);
		s->SetUniform("mat_bloom_hdr_ssao","rad",0.1);
		s->SetUniform("mat_bloom_hdr_ssao","bias",0.07);
	}
	s->PostInit();	
}

