/**
****************************************************************************************************
****************************************************************************************************
@file scenes.h
@brief initialization and running of demo
****************************************************************************************************
***************************************************************************************************/
#include "demo.h"

/**
**************************************************************************************************
@brief Run all scenes
**************************************************************************************************/
void TDemo::RunScenes(int fps)
{
	anim = 30.0/fps;

	if(current_scene == INTRO)
	{
		//animate text
		if(local_time > 1.0 && local_time < 2.0)
			s->MoveObjAbs("intro1",30.0*local_time,0.0,0.0);
	  if(local_time > 2.5 && local_time < 4.0)
		{
			exposure = 0.5 + 2.0*abs(sin( 2.0*(local_time - 2.5)));
			s->MoveObjAbs("introtext",0.0, 8.0*local_time,0.0);
		}
		if(local_time > 4.5 && local_time < 5.5)
			s->MoveObjAbs("intro2", -30.0*local_time + 30.0,0.0,0.0);
		if(local_time > 7.0)
		{
			drawtext = "Texel fillrate";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 8.0)
			NextScene();
	}
  else if(current_scene == FILLRATE)
	{
		//lower exposure
		if(local_time < 1.0)
		{
			if(exposure < 1.5)
				exposure += anim/20.0;
		}
		//animate surfaces
		for(int i = 0; i < surfaces; i++)
		{
			obj_name = "surface1" + num2str(i);
			s->RotateObj(obj_name.c_str(),anim/100.0 + i/200.0,A_Y);
			obj_name = "surface2" + num2str(i);
			s->RotateObj(obj_name.c_str(),anim/100.0 + i/200.0,A_Y);
		}
		//move camera
		s->RotateCamera(anim/10.0,A_Y);
		//end scene, fade to black
		if(local_time > 8.0)
		{
			drawtext = "Per-pixel lighting with normal mapping";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 9.0)
			NextScene();
	}
	else if(current_scene == FRAG_LIGHTS)
	{
			//adjust exposure
			if(local_time < 1.0)
			{
				if(exposure < 0.8)
					exposure += anim/20.0;
			}
			//light movement
			lpos1.x = 10.0f*cos(local_time);
			lpos1.z = 45.0f*sin(local_time);
			s->MoveLight(0,lpos1);

			lpos2.y = 10.0f*cos(1.5f*local_time) + 10.0f;
			lpos2.z = 45.0f*sin(1.5f*local_time);
			s->MoveLight(1,lpos2);

			lpos3.x = 10.0f*sin(local_time) - 10.0f;
			lpos3.y = 10.0f*cos(0.5f*local_time) + 10.0f;
			lpos3.z = 45.0f*cos(0.5f*local_time);
			s->MoveLight(2,lpos3);

			lpos4.x = 10.0f*sin(0.4f*local_time) + 10.0f;
			lpos4.y = 10.0f*sin(0.7f*local_time) + 10.0f;
			lpos4.z = 45.0f*sin(0.7f*local_time);
			s->MoveLight(3,lpos4);

			lpos5.x = 20.0f*cos(1.3f*local_time);
			lpos5.z = 20.0f*sin(1.3f*local_time);
			s->MoveLight(4,lpos5);

			lpos6.y = 10.0f*cos(1.8f*local_time) + 10.0f;
			lpos6.z = 10.0f*sin(1.8f*local_time);
			s->MoveLight(5,lpos6);

			lpos7.x = 10.0f*sin(1.6f*local_time) - 10.0f;
			lpos7.y = 10.0f*cos(1.6f*local_time) + 10.0f;
			lpos7.z = 10.0f*cos(1.6f*local_time);
			s->MoveLight(6,lpos7);

			lpos8.x = 10.0f*sin(0.9f*local_time) + 10.0f;
			lpos8.y = 10.0f*sin(0.8f*local_time) + 10.0f;
			lpos8.z = 10.0f*sin(0.8f*local_time);
			s->MoveLight(7,lpos8);

			//end scene, fade
			if(local_time > 8.0)
			{
				drawtext = "Procedural materials";
				if(exposure > 0.0) 
					exposure -= anim/20.0;
			}
			if(local_time > 9.0)
				NextScene();

			s->RotateCameraAbs(local_time*3.3, A_Y);
	}
	else if(current_scene == FRAG_PROCEDURAL)
	{
			s->RotateCameraAbs(-local_time*30.0, A_Y);

			//adjust exposure
			if(local_time < 1.0)
			{
				if(exposure < 0.5)
					exposure += anim/20.0;
			}
		  //animate noise
			s->SetUniform("mat_proc_bump","anim",local_time/5.0);
			s->SetUniform("mat_proc_sky","anim",local_time/10.0);
			//end scene, fade
			if(local_time > 8.0)
			{
				drawtext = "Parallax mapping";
				if(exposure > 0.0) 
					exposure -= anim/20.0;
			}
			if(local_time > 9.0)
				NextScene();
	}
	else if(current_scene == FRAG_PARALLAX)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure < 1.0)
				exposure += anim/20.0;
		}
		//light movement
    lpos1.x = 10.0f*cos(local_time);
    lpos1.y = 10.0f*sin(local_time);
		s->MoveLight(0,lpos1);
		//end scene, fade
		if(local_time > 8.0)
		{
			drawtext = "Refraction with chromatic dispersion";
			if(exposure < 10.0) 
				exposure += anim/5.0;
		}
		if(local_time > 9.0)
			NextScene();
	}
	else if(current_scene == FRAG_REFRACT)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure > 1.0)
				exposure -= anim/5.0;
		}
		s->RotateCameraAbs(local_time*30.0, A_Y);
		s->RotateCameraAbs(local_time*20.0, A_X);
		//end scene, fade
		if(local_time > 10.0)
		{
			drawtext = "Dynamic reflections";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 11.0)
			NextScene();
	}
	else if(current_scene == GEOM_CUBEMAP)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure < 1.0)
				exposure += anim/20.0;
		}
    //spaceships movement
		object_count = 7;
    for(int i=0; i<object_count*object_count; i++)
    {
      obj_name = "fighter" + num2str(i);
      s->MoveObj(obj_name.c_str(), -speeds[i]/fps*50.0f,0.0f,0.0f);
      if(s->GetObjPosition(obj_name.c_str()).x < -40.0)
        s->MoveObj(obj_name.c_str(), 80.0, 0.0, 0.0);
    }
    s->MoveObjAbs("rflx_sphere2",sin(local_time/5.0f + 4.0f) * 2.0f, 0.0f, cos(local_time/5.0f + 4.0f) * 2.0f);
		s->RotateCameraAbs(local_time*30.0, A_Y);
		s->RotateCameraAbs(sin(local_time/2.0)*10.0, A_X);
		s->MoveCamera(0.0,0.0,anim/50.0);
		//end scene, fade
		if(local_time > 9.0)
		{
			drawtext = "Geometry shader particles";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 11.0)
			NextScene();
	}
	else if(current_scene == GEOM_PARTICLES)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure < 0.7)
				exposure += anim/20.0;
		}
		s->SetUniform("mat_rain","time",local_time/3.0);
    s->SetUniform("mat_wet","time",local_time/3.0);
		//end scene, fade
		if(local_time > 8.0)
		{
			drawtext = "Per-vertex lighting with geometry instancing";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 9.0)
			NextScene();
		s->MoveCamera(0.0,0.0,-anim/100.0);
	}
	else if(current_scene == VERT_LIGHTS)
	{
		//adjust exposure
		if(local_time < 3.0)
		{
			if(exposure < 2.0)
				exposure += anim/20.0;
		}
		//animate lights
    lpos1.x = 10.0f*cos(local_time);
    lpos1.z = 45.0f*sin(local_time);
    lpos2.y = 10.0f*cos(1.5f*local_time) + 10.0f;
    lpos2.z = 45.0f*sin(1.5f*local_time);
    lpos3.x = 10.0f*sin(local_time) - 10.0f;
    lpos3.y = 10.0f*cos(0.5f*local_time) + 10.0f;
    lpos3.z = 45.0f*cos(0.5f*local_time);
    lpos4.x = 10.0f*sin(0.4f*local_time) + 10.0f;
    lpos4.y = 10.0f*sin(0.7f*local_time) + 10.0f;
    lpos4.z = 45.0f*sin(0.7f*local_time);
		s->MoveLight(0,lpos1);
    s->MoveLight(1,lpos2);
    s->MoveLight(2,lpos3);
    s->MoveLight(3,lpos4);

		s->RotateCameraAbs(local_time*20.0, A_Y);
		s->RotateCameraAbs(local_time, A_X);
		s->MoveCameraAbs(0.0f,-1.0f,-20.0f - local_time/3.0f);

		//end scene, fade
		if(local_time > 10.0)
		{
			drawtext = "Procedurally generated waves";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 11.0)
			NextScene();
	}
	else if(current_scene == VERT_WAVES)
	{
		//adjust exposure
		if(local_time < 3.0)
		{
			if(exposure < 0.8)
				exposure += anim/10.0;
		}
		s->RotateCameraAbs(local_time*9.0f,A_Y);
		s->SetUniform("mat_sea","time",local_time);	//update animation
		//end scene, fade
		if(local_time > 8.0)
		{
			drawtext = "Vertex displacement mapping";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 9.0)
			NextScene();
	}
	else if(current_scene == VERT_DISPLACE)
	{
		//adjust exposure
		if(local_time < 3.0)
		{
			if(exposure < 1.0)
				exposure += anim/20.0;
		}
		s->MoveCameraAbs(-1.4f,-8.0f,-90.0f + local_time*7.0);
		s->SetUniform("disp_mat_terrain","disp_mat_terrainDisplaceA_intensity",local_time*8.0);
		//end scene, fade
		if(local_time > 8.0)
		{
			drawtext = "HDR lighting";
			if(exposure < 30.0) 
				exposure += anim/10.0;
		}
		if(local_time > 10.0)
			NextScene();
	}
	else if(current_scene == COMPLEX_HDR)
	{
		//adjust exposure
		if(local_time < 3.0)
		{
			if(exposure > 0.9)
				exposure -= anim/10.0;
		}
		//keyframe diff
		if(local_time > 2.0)
		{
			dp = (key2_pos - key1_pos)/(float)fps/12.0f * (float)inv;
			dr = (key2_rot - key1_rot)/(float)fps/12.0f * (float)inv;

			s->MoveCamera(dp.x, dp.y, dp.z);
			s->RotateCamera(dr.x, A_X);
			s->RotateCamera(dr.y, A_Y);
		}
		//end scene, fade
		if(local_time > 15.0)
		{
			drawtext = "Screen space ambient occlusion";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 16.0)
			NextScene();
	}
	else if(current_scene == COMPLEX_SSAO)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure < 1.0)
				exposure += anim/20.0;
		}
		//animation
		s->RotateCameraAbs(local_time*10.0f,A_Y);
		s->MoveObjAbs("motor",0.0f,sin(5.0f*local_time), 0.0f);
		s->MoveObjAbs("motor_fan",-32.5f, 14.0f + sin(5.0f*local_time), -4.5f);
		s->RotateObjAbs("motor_fan",local_time*100.0f,A_X);
		//end scene, fade
		if(local_time > 10.0)
		{
			drawtext = "Dynamic soft shadows";
			if(exposure > 0.0) 
				exposure -= anim/20.0;
		}
		if(local_time > 11.0)
			NextScene();
	}
	else if(current_scene == COMPLEX_SHADOW)
	{
		//move light and change intensity
		float t_anim = PI/2.0f + local_time/5.0;
		lpos1.z = 50.0f * sin(t_anim);
		lpos1.y = 30.0f * abs(cos(t_anim)) + 10.0f;
		s->MoveLight(0,lpos1);
		exposure = abs(cos(t_anim));
		//end scene, fade
		if(local_time > 14.5)
			drawtext = "OpenCL N-Body simulation";
		if(local_time > 15.5)
			NextScene();
	}
	else if(current_scene == COMPUTE_NBODY)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure < 1.7)
				exposure += anim/20.0;
		}
		//calculate new body positions through OpenCL
		nbody->Run();

		body = 0;
		for(int i=0; i<halfbody; i++)
		{
			sprintf(objname,"1_star%d",i);
			s->MoveObjAbs(objname,nbody->GetPos(body*4 + 0), nbody->GetPos(body*4 + 1), nbody->GetPos(body*4 + 2));
			body++;
		}
		for(int i=0; i<halfbody; i++)
		{
			sprintf(objname,"2_star%d",i);
			s->MoveObjAbs(objname,nbody->GetPos(body*4 + 0), nbody->GetPos(body*4 + 1), nbody->GetPos(body*4 + 2));
			body++;
		}
		s->RotateCameraAbs(local_time*5.0f, A_Y);
		//end scene, fade
		if(local_time > 10.0)
		{
			drawtext = "";
			if(exposure > 0.0) 
				exposure -= anim/10.0;
		}
		if(local_time > 11.0)
			NextScene();
	}
	else if(current_scene == OUTRO)
	{
		//adjust exposure
		if(local_time < 2.0)
		{
			if(exposure < 0.8)
				exposure += anim/20.0;
		}
		s->RotateCameraAbs(local_time*5.0f, A_Y);
	}

	s->SetUniform("mat_tonemap","exposure", exposure);
	s->Redraw();
	//string expo = "exposure: " + num2str(exposure);
	//s->DrawScreenText(expo.c_str(),90,1);

	//animate drawtext
	s->DrawScreenText(drawtext.c_str(),50,50 + local_time*7,4.0 - local_time/2.5);

	//outro - subtitles
	if(current_scene == OUTRO)
	{
		float subpos = local_time * 5;
		if(subpos > 50.0)
			subpos = 50.0;

		s->DrawScreenText("gluxMark2 DEMO",50,subpos + 15,2.0);
		s->DrawScreenText("GFX by Juraj \"jammer\" Vanek",50,subpos + 5,1.5);
		s->DrawScreenText("Music by Anton \"tony\" Vanek",50,subpos - 5,1.5);
		s->DrawScreenText("Thanks for watching :)",50,subpos - 15,1.5);
	}
}

