///-----------------------------------------------------------------
///
/// @file      gui2.h
/// @author    Juraj Vanek
/// Created:   15. 4. 2010 23:44:06
/// @section   DESCRIPTION
///            results parser
///
///------------------------------------------------------------------
#include "gui.h"

/* 
 * Parse system information from XML file
 */
bool TForm::ParseSystemInfo()
{
  CLsupport = true;

#ifdef _WIN_  
  //first, find out if we have OpenCL support
  if(LoadLibrary("OpenCL.dll") != NULL)
  {
    CLsupport = true;
    app_exec = wxT("bin/bench.exe");
  }
  else
  {
    CLsupport = false;
    app_exec = wxT("bin/bench_noCL.exe");
  }
#else
  app_exec = wxT("bin/bench");
#endif
  //run application with -info
  long ret_code = wxExecute(app_exec + wxT(" -t info"),wxEXEC_SYNC);
  if(ret_code == -1)  //unsupported HW
  {
    wxMessageBox(wxT("Your video card doesn't meet minimum requirements.\nYou won't be able to run any of the test :("));
    //disable RUN button
    btnRun->Disable();
  }
  wxXmlDocument sysinfoXML;
  //load file
  if(!sysinfoXML.Load(wxT("sysinfo.xml")))
  {
    wxMessageBox(wxT("Cannot parse system info!"),wxT("Error"),wxICON_ERROR);
    return false;
  }
  GSsupport = true;
  //go through whole file and put results to HTML page
  wxXmlNode *root = sysinfoXML.GetRoot()->GetChildren();
  wxXmlNode *child;
  root = root->GetChildren();                
  while (root) 
  {
    if(root->GetName() == wxT("system"))    
    {
      child = root->GetChildren();
      while(child)
      {
        if (child->GetName() == wxT("os"))
        {
          sysInfoHTML<<wxT("<font color=\"teal\" size=\"4\"><i><b>System info:</b></i></font><table>");
          sysInfoHTML<<wxT("<tr><td><b>OS:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
          textOS->SetLabel(child->GetNodeContent());
        }
        else if (child->GetName() == wxT("cpu"))
        {
          sysInfoHTML<<wxT("<tr><td><b>CPU:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
          textCPU->SetLabel(child->GetNodeContent());
        }
        else if (child->GetName() == wxT("cpufreq"))
          sysInfoHTML<<wxT("<tr><td><b>CPU real frequency:</b></td><td>")<<child->GetNodeContent()<<wxT(" MHz</td></tr>\n");
        else if (child->GetName() == wxT("memory"))
          sysInfoHTML<<wxT("<tr><td><b>System RAM:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr></table><br><br>\n");
        child = child->GetNext();  
      }
    }
    if(root->GetName() == wxT("gfx"))
    {
      child = root->GetChildren();
      while(child)
      {
        if (child->GetName() == wxT("name"))
        {
          sysInfoHTML<<wxT("<font color=\"teal\" size=\"4\"><i><b>GFX info:</b></i></font>\n<table>\n");
          sysInfoHTML<<wxT("<tr><td><b>Graphic card:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
          textGFX->SetLabel(child->GetNodeContent());
        }
        else if (child->GetName() == wxT("vendor"))
          sysInfoHTML<<wxT("<tr><td><b>Manufacturer:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr\n>");
        else if (child->GetName() == wxT("vram"))
          sysInfoHTML<<wxT("<tr><td><b>Video RAM:</b></td><td>")<<child->GetNodeContent()<<wxT(" MB</td></tr>\n");       
        else if (child->GetName() == wxT("glversion"))
        {
          sysInfoHTML<<wxT("<tr><td><b>OpenGL version:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
          textOGL->SetLabel(child->GetNodeContent());
        }
        else if (child->GetName() == wxT("glslversion"))
          sysInfoHTML<<wxT("<tr><td><b>GLSL version:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");   
        else if (child->GetName() == wxT("extensions"))
          sysInfoHTML<<wxT("<tr><td><b>OpenGL extensions:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr></table><br><br>\n");  
         else if (child->GetName() == wxT("hasMRT"))
        {
          sysInfoHTML<<wxT("<font color=\"teal\" size=\"4\"><i><b>OpenGL capabilities:</b></i></font>\n<table>\n");
          sysInfoHTML<<wxT("<tr><td><b>Multiple render targets:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        }             
        else if (child->GetName() == wxT("hasGI"))
          sysInfoHTML<<wxT("<tr><td><b>Geometry instancing:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");       
        else if (child->GetName() == wxT("hasGS"))
        {
          sysInfoHTML<<wxT("<tr><td><b>Geometry shaders:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n"); 
          //no GS support? disable test selection
          if(child->GetNodeContent() == wxT("NO"))
            GSsupport = false;          
        }
        else if (child->GetName() == wxT("hasFloat"))
          sysInfoHTML<<wxT("<tr><td><b>Floating-point textures:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
        else if (child->GetName() == wxT("hasFBOMulti"))
          sysInfoHTML<<wxT("<tr><td><b>Multisampled framebuffer:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");                                                      
        else if (child->GetName() == wxT("texsize"))
        {
          sysInfoHTML<<wxT("<tr><td><font color=\"teal\" size=\"3\"><i><b>Renderer features:</b></i></font></td><td></td></tr>\n");
          sysInfoHTML<<wxT("<tr><td><b>2D texture size:</b></td><td>")<<child->GetNodeContent()<<wxT(" px</td></tr>\n");
        }
        else if (child->GetName() == wxT("texsize3D"))
          sysInfoHTML<<wxT("<tr><td><b>3D texture size:</b></td><td>")<<child->GetNodeContent()<<wxT(" px</td></tr>\n");
        else if (child->GetName() == wxT("tex_layers"))
          sysInfoHTML<<wxT("<tr><td><b>Texture layers:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("cubetexsize"))
          sysInfoHTML<<wxT("<tr><td><b>Cube texture size:</b></td><td>")<<child->GetNodeContent()<<wxT(" px</td></tr>\n");
        else if (child->GetName() == wxT("renderbufsize"))
          sysInfoHTML<<wxT("<tr><td><b>Renderbuffer size:</b></td><td>")<<child->GetNodeContent()<<wxT(" px</td></tr>\n");   
        else if (child->GetName() == wxT("drawbuffs"))
          sysInfoHTML<<wxT("<tr><td><b>Multiple render targets:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n"); 
        else if (child->GetName() == wxT("msamples"))
          sysInfoHTML<<wxT("<tr><td><b>Multisample count:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
        else if (child->GetName() == wxT("indices"))
          sysInfoHTML<<wxT("<tr><td><b>Index buffer size:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");       
        else if (child->GetName() == wxT("vertices"))
          sysInfoHTML<<wxT("<tr><td><b>Vertex buffer size:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");    
          
         else if (child->GetName() == wxT("comb_texunits"))
         {
          sysInfoHTML<<wxT("<tr><td><font color=\"teal\" size=\"3\"><i><b>Shader features:</b></i></font></td><td></td></tr>\n");
          sysInfoHTML<<wxT("<tr><td><b>Combined texture units:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n"); 
        }  
         else if (child->GetName() == wxT("varyings"))
          sysInfoHTML<<wxT("<tr><td><b>Varying variables:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
         else if (child->GetName() == wxT("vertattribs"))
          sysInfoHTML<<wxT("<tr><td><b>Vertex attributes:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
         else if (child->GetName() == wxT("vertunifs"))
          sysInfoHTML<<wxT("<tr><td><b>Vertex uniform components:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
         else if (child->GetName() == wxT("gsout_vertices"))
          sysInfoHTML<<wxT("<tr><td><b>Emitted vertices by geometry shader:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
         else if (child->GetName() == wxT("gsout_components"))
          sysInfoHTML<<wxT("<tr><td><b>Total geometry shader outputs:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");  
         else if (child->GetName() == wxT("fragunifs"))
          sysInfoHTML<<wxT("<tr><td><b>Fragment uniform components:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr></table><br><br>\n");                                                    
                                                                    
        child = child->GetNext();        
      }
    }    
    if(root->GetName() == wxT("opencl"))    
    {
      child = root->GetChildren();
      while(child)
      {
        if (child->GetName() == wxT("isCL"))
        {
          sysInfoHTML<<wxT("<font color=\"teal\" size=\"4\"><i><b>OpenCL info:</b></i></font><table>");
          sysInfoHTML<<wxT("<tr><td><b>OpenCL support:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
          //no OpenCL support? disable test selection
          if(child->GetNodeContent() == wxT("NO"))
          {
            CLsupport = false;
            //change application executable
#ifdef _WIN_
            app_exec = wxT("bin/bench_noCL.exe");
#else
            app_exec = wxT("bin/bench_noCL");
#endif              
            break;
          }
        }
        else if (child->GetName() == wxT("vendor"))
          sysInfoHTML<<wxT("<tr><td><b>Vendor:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("device"))
          sysInfoHTML<<wxT("<tr><td><b>Device name:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");   
        else if (child->GetName() == wxT("units"))
          sysInfoHTML<<wxT("<tr><td><b>Compute units:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("clock"))
          sysInfoHTML<<wxT("<tr><td><b>Core clock rate:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("isLocal"))
          sysInfoHTML<<wxT("<tr><td><b>Local memory support:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("globMem"))
          sysInfoHTML<<wxT("<tr><td><b>Global memory size:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");                                               
        else if (child->GetName() == wxT("workitems"))
          sysInfoHTML<<wxT("<tr><td><b>Max. work item sizes:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");        
        else if (child->GetName() == wxT("workgroup"))
          sysInfoHTML<<wxT("<tr><td><b>Max. work group size:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr></table>\n");        
        child = child->GetNext();  
      }
    }    
    root = root->GetNext();
  }
  sysInfoHTML<<wxT("</table>");
  sysInfo->SetData(sysInfoHTML);
  
  //update GUI
  changeDialog->SetSupport(GSsupport,CLsupport);
  
  return true;
}






















/*
 * Parse results from XML file generated by benchmark
 */
bool TForm::ParseResults()
{
  wxXmlDocument resultsXML;

  //load file
  if(!resultsXML.Load(wxT("results.xml")))
  {
    /*
    //ak sa nepodari, skusime este v temp subore
#ifdef _WIN_    
    if(!resultsXML.Load(tmpath + wxT("\\results.xml")))
#else
    if(!resultsXML.Load(tmpath + wxT("/results.xml")))
#endif      
*/
      return false;
  }  

  wxXmlNode *root = resultsXML.GetRoot()->GetChildren(); 
  wxXmlNode *child,*child2,*tmp;

  //begin creation of webpage
  wxString html_head, html_core, html_tail;
  //some variables used to parse score
  double fps, fillrate, vertrate, score, graph_fps, graph,
    fill_score = 0, frag_score = 0, vert_score = 0, geom_score = 0, complex_score = 0, compute_score = 0;

  html_core = wxT("   <div class=\"obsah\">\n");
  const int graph_width = 550;

  //system info
  html_tail<<wxT(
//system info
"   <table align=\"center\" class=\"info\">\n"
"   <tr><th colspan=\"2\"><a name=\"info\">SYSTEM INFO</th></tr>\n"
"   <tr><td>")<<sysInfoHTML<<wxT("</td></tr>\n"
"   </table>\n");
                                                
  while (root)
  {
    //benchmark settings
    if(root->GetName() == wxT("settings"))
    {
      child = root->GetChildren();
      while(child)
      {
        if(child->GetName() == wxT("resolution")) 
          html_tail<<wxT(
            "<table align=\"center\" class=\"info\">\n"
            "   <tr><th colspan=\"2\">BENCHMARK SETTINGS</th></tr>\n"
            "   <tr><td><b>Resolution:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("antialias")) 
          html_tail<<wxT("   <tr><td><b>Antialiasing:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n");
        else if (child->GetName() == wxT("series")) 
          html_tail<<wxT("   <tr><td><b>Test series:</b></td><td>")<<child->GetNodeContent()<<wxT("</td></tr>\n</table>\n");                  
        child = child->GetNext();
      }      
    }
    
    //*********************************************************************************************
    //FILLRATE TESTS
    if(root->GetName() == wxT("fillrate"))     
    {
      child = root->GetChildren();    
      //SINGLE TEXTURING
      while(child)
      {
        if (child->GetName() == wxT("texel_fillrate_single"))
        {    
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&fillrate); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          fill_score += score;
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;
          graph = fillrate/50.0; 
          if(graph > graph_width) graph=graph_width;                  
          //write         
          html_core<<wxT( 
            "<table align=\"center\">\n"
            "<tr><th colspan=\"2\">FILLRATE TESTS RESULTS</th></tr>\n"
            "<tr><td class=\"screen\"><img src=\"www/pic/test01.jpg\"></td>\n" 
            "<td><h2>Texel fillrate(single texturing)<a name=\"fillrate\"></h2>\n"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<fillrate<<wxT("&nbsp;MTexels/s</b></span>\n"
            "</td></tr>\n");  
        }
        
        //MULTI TEXTURING
        else if (child->GetName() == wxT("texel_fillrate_multi"))
        {
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&fillrate); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          fill_score += score;   
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;
          graph = fillrate/50.0; 
          if(graph > graph_width) graph=graph_width;   
          //write         
          html_core<<wxT( 
            "<tr><td class=\"screen\"><img src=\"www/pic/test02.jpg\"></td>\n" 
            "<td><h2>Texel fillrate(multi texturing)</h2>\n"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"
            " <span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<fillrate<<wxT("&nbsp;MTexels/s</b></span>\n"
            "</td></tr>\n");                                
        }
        
        //HDR 16-BIT
        else if (child->GetName() == wxT("texel_fillrate_float16"))
        {
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&fillrate); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          fill_score += score;   
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;
          graph = fillrate/50; 
          if(graph > graph_width) graph=graph_width;   
          //write         
          html_core<<wxT( 
            "<tr><td class=\"screen\"><img src=\"www/pic/test03.jpg\"></td>\n" 
            "<td><h2>Texel fillrate(16-bit floating point textures)</h2>\n"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<fillrate<<wxT("&nbsp;MTexels/s</b></span>\n"
            "</td></tr>\n");   
        }
        
        //HDR 32-BIT
        else if (child->GetName() == wxT("texel_fillrate_float32"))
        {
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&fillrate); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          fill_score += score;   
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;
          graph = fillrate/50; 
          if(graph > graph_width) graph=graph_width;   
          //write         
          html_core<<wxT( 
            "<tr><td class=\"screen\"><img src=\"www/pic/test04.jpg\"></td>\n" 
            "<td><h2>Texel fillrate(32-bit floating point textures)</h2>\n"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<fillrate<<wxT("&nbsp;MTexels/s</b></span>\n"
            "</td></tr>\n</table><hr>");   
        }                
        child = child->GetNext();
      }
    }
    
    //*********************************************************************************************
    //FRAGMENT SHADERS
    if(root->GetName() == wxT("fragment"))     //fragment shader tests
    {
      child = root->GetChildren();    
      while(child)
      {
        
        //PERPIXEL LIGHTS
        if (child->GetName() == wxT("frag_lights"))
        {  
          html_core<<wxT(
            "<table align=\"center\">\n"
            "<tr><th colspan=\"2\">FRAGMENT SHADER TESTS RESULTS</th></tr>\n"          
            "<tr><td class=\"screen\"><img src=\"www/pic/test05.jpg\"></td>\n" 
            "<td><h2>Per-pixel Phong lighting<a name=\"fragment\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every light
          {
            wxString name;
            if(child2->GetName() == wxT("light1")) name = wxT("1 light");
            else if(child2->GetName() == wxT("light4")) name = wxT("4 lights");
            else if(child2->GetName() == wxT("light8")) name = wxT("8 lights");
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              frag_score += score;     
              break;
            }
            
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");           
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }     
        
        //PERLIN NOISE
        else if(child->GetName() == wxT("frag_noise"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test06.jpg\"></td>\n" 
            "<td><h2>Perlin noise</h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every octave
          {
            wxString name;
            if(child2->GetName() == wxT("oct2")) name = wxT("2 octaves");
            else if(child2->GetName() == wxT("oct4")) name = wxT("4 octaves");
            else if(child2->GetName() == wxT("oct8")) name = wxT("8 octaves");
            else if(child2->GetName() == wxT("oct16")) name = wxT("16 octaves");            
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              frag_score += score;     
              break;
            }
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");           
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }      
        
        //PROCEDURAL SHADERS
        else if(child->GetName() == wxT("frag_procedural"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test07.jpg\"></td>\n" 
            "<td><h2>Procedural shaders</h2>\n");      
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          frag_score += score;            
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;   
          //write
          html_core<< wxT(
          "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n</td></tr>\n");                        
        }     
        
        //PARALLAX MAPPING
        else if(child->GetName() == wxT("frag_parallax"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test08.jpg\"></td>\n" 
            "<td><h2>Parallax mapping</h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("perpixel")) name = wxT("Regular per-pixel lighting");
            else if(child2->GetName() == wxT("bump")) name = wxT("Bump(normal) mapping");
            else if(child2->GetName() == wxT("parallax")) name = wxT("Parallax mapping");           
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              frag_score += score;     
              break;
            }
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");           
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }           
   
        //REFRACTION/REFLECTION
        else if(child->GetName() == wxT("frag_refract"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test09.jpg\"></td>\n" 
            "<td><h2>Static reflection & refraction</h2>\n");      
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("ref_custom")) name = wxT("Custom refract function");
            else if(child2->GetName() == wxT("ref_builtin")) name = wxT("Built-in refract function");
            else if(child2->GetName() == wxT("ref_custom_chroma")) name = wxT("Custom refract function with chromatic diffusion");           
            else if(child2->GetName() == wxT("ref_builtin_chroma")) name = wxT("Built-in refract with chromatic diffusion");                 
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              frag_score += score;     
              break;
            }
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");           
            child2 = child2->GetNext();        
          }      
          html_core<<wxT("</td></tr>\n</table><hr>");
        }                                
        child = child->GetNext();  
      }                    
    }
    
    //*********************************************************************************************
    //GEOMETRY SHADERS
    if(root->GetName() == wxT("geometry"))
    {
      child = root->GetChildren();    
      while(child)
      {
        
        //TESSELATION
        if (child->GetName() == wxT("geom_tesselate"))
        {  
          html_core<<wxT(
            "<table align=\"center\">\n"
            "<tr><th colspan=\"2\">GEOMETRY SHADER TESTS RESULTS</th></tr>\n"          
            "<tr><td class=\"screen\"><img src=\"www/pic/test10.jpg\"></td>\n" 
            "<td><h2>Tesselation with depth map<a name=\"geometry\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("level0")) name = wxT("Tesselation level 1");
            else if(child2->GetName() == wxT("level1")) name = wxT("Tesselation level 2");
            else if(child2->GetName() == wxT("level2")) name = wxT("Tesselation level 3");
            else if(child2->GetName() == wxT("level3")) name = wxT("Tesselation level 4");            
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              geom_score += score;     
              break;
            }
            
            //Get FPS & MVertices
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            tmp = tmp->GetNext();
            tmp->GetNodeContent().ToDouble(&vertrate);            
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            graph = 2*vertrate;
            if(graph > graph_width) graph=graph_width;              
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"     
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<vertrate<<wxT("&nbsp;MVertices/s</b></span>\n");                   
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }     
        
        //GS CUBEMAPPING
        else if(child->GetName() == wxT("geom_cubemap"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test11.jpg\"></td>\n" 
            "<td><h2>Dynamic single-pass reflections</h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("GS_off")) name = wxT("Multi-pass cubemapping");
            else if(child2->GetName() == wxT("GS_on")) name = wxT("Single-pass cubemapping");         
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              geom_score += score;     
              break;
            }
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");           
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }        
        
        //GPU PARTICLES
        else if(child->GetName() == wxT("geom_particles"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test12.jpg\"></td>\n" 
            "<td><h2>GPU particles</h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("part_1")) name = wxT("1600 particles");
            else if(child2->GetName() == wxT("part_2")) name = wxT("6400 particles");
            else if(child2->GetName() == wxT("part_3")) name = wxT("14 400 particles"); 
            else if(child2->GetName() == wxT("part_4")) name = wxT("25 600 particles");             
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              geom_score += score;     
              break;
            }
            //Get FPS & particle count
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            tmp = tmp->GetNext();
            tmp->GetNodeContent().ToDouble(&vertrate);            
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            graph = vertrate/2000.0;
            if(graph > graph_width) graph=graph_width;              
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"     
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<vertrate<<wxT("&nbsp;Particles/s</b></span>\n");                   
            child2 = child2->GetNext();                
          }      
          html_core<<wxT("</td></tr>\n");
        }           
   
        //GS INSTANCING
        else if(child->GetName() == wxT("geom_instancing"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test13.jpg\"></td>\n" 
            "<td><h2>Geometry shader instancing</h2>\n");      
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("inst_off")) name = wxT("Instancing OFF");
            else if(child2->GetName() == wxT("inst_on")) name = wxT("Instancing ON");               
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              geom_score += score;     
              break;
            }
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");           
            child2 = child2->GetNext();        
          }      
          html_core<<wxT("</td></tr>\n</table><hr>");
        }   
        child = child->GetNext();  
      }        
    }
  
    //*********************************************************************************************
    //VERTEX SHADERS
    if(root->GetName() == wxT("vertex"))
    {
      child = root->GetChildren();    
      while(child)
      {
        
        //PER-VERTEX LIGHTING
        if (child->GetName() == wxT("vert_lights"))
        {  
          html_core<<wxT(
            "<table align=\"center\">\n"
            "<tr><th colspan=\"2\">VERTEX SHADER TESTS RESULTS</th></tr>\n"          
            "<tr><td class=\"screen\"><img src=\"www/pic/test14.jpg\"></td>\n" 
            "<td><h2>Per-vertex lighting with geometry instancing<a name=\"vertex\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("light1")) name = wxT("1 light");
            else if(child2->GetName() == wxT("light4")) name = wxT("4 lights");
            else if(child2->GetName() == wxT("light8")) name = wxT("8 lights");         
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              vert_score += score;     
              break;
            }
            
            //Get FPS & MVertices
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            tmp = tmp->GetNext();
            tmp->GetNodeContent().ToDouble(&vertrate);            
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            graph = vertrate;
            if(graph > graph_width) graph=graph_width;              
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"     
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<vertrate<<wxT("&nbsp;MVertices/s</b></span>\n");                   
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }     
        
        //VERTEX WAVES
        else if(child->GetName() == wxT("vert_waves"))
        {  
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&vertrate); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          vert_score += score;   
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;
          graph = vertrate; 
          if(graph > graph_width) graph=graph_width;   
          //write         
          html_core<<wxT( 
            "<tr><td class=\"screen\"><img src=\"www/pic/test15.jpg\"></td>\n" 
            "<td><h2>Procedural wave vertex shader</h2>\n"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"
            " <span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<vertrate<<wxT("&nbsp;MVertices/s</b></span>\n"
            "</td></tr>\n"); 
        }        
        
        //DISPLACEMENT MAPPING
        else if(child->GetName() == wxT("vert_displace"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test16.jpg\"></td>\n" 
            "<td><h2>Vertex displacement mapping</h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("disp_off")) name = wxT("Displacement OFF");
            else if(child2->GetName() == wxT("disp_on")) name = wxT("Displacement ON");         
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              vert_score += score;     
              break;
            }
            //Get FPS & vertex count
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);
            tmp = tmp->GetNext();
            tmp->GetNodeContent().ToDouble(&vertrate);            
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;   
            graph = vertrate;
            if(graph > graph_width) graph=graph_width;              
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"     
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<vertrate<<wxT("&nbsp;MVertices/s</b></span>\n");                   
            child2 = child2->GetNext();                
          }           
          html_core<<wxT("</td></tr>\n</table><hr>");
        }   
        child = child->GetNext();  
      }                
    }

    //*********************************************************************************************
    //COMPLEX TESTS
    if(root->GetName() == wxT("complex"))
    {
      child = root->GetChildren();    
      while(child)
      {
        
        //HDR LIGHTING
        if (child->GetName() == wxT("complex_hdr"))
        {  
          html_core<<wxT(
            "<table align=\"center\">\n"
            "<tr><th colspan=\"2\">HDR/COMPLEX TESTS RESULTS</th></tr>\n"          
            "<tr><td class=\"screen\"><img src=\"www/pic/test17.jpg\"></td>\n" 
            "<td><h2>HDR lighting<a name=\"complex\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("hdr_off")) name = wxT("HDR OFF");
            else if(child2->GetName() == wxT("hdr_on")) name = wxT("HDR ON");      
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              complex_score += score;     
              break;
            }
            
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);          
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;                
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");                  
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }     
        
        //SSAO
        if (child->GetName() == wxT("complex_ssao"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test18.jpg\"></td>\n" 
            "<td><h2>Screen space ambient occlusion (SSAO)<a name=\"complex\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("ao_off")) name = wxT("SSAO OFF");
            else if(child2->GetName() == wxT("ao_on")) name = wxT("SSAO ON");      
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              complex_score += score;     
              break;
            }
            
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);          
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;                
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");                  
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }  
        
        //DYNAMIC SHADOWS
        if (child->GetName() == wxT("complex_shadow"))
        {  
          html_core<<wxT(        
            "<tr><td class=\"screen\"><img src=\"www/pic/test19.jpg\"></td>\n" 
            "<td><h2>Dynamic soft shadows<a name=\"complex\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("shadow_off")) name = wxT("Shadows OFF");
            else if(child2->GetName() == wxT("shadow_on")) name = wxT("Shadows ON");      
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              complex_score += score;     
              break;
            }
            
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);          
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;                
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");                  
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n");
        }  
        
        //SUPERSAMPLING
        if (child->GetName() == wxT("complex_ssampling"))
        {  
          html_core<<wxT(         
            "<tr><td class=\"screen\"><img src=\"www/pic/test20.jpg\"></td>\n" 
            "<td><h2>Render with supersampling<a name=\"complex\"></h2>\n");          
          child2 = child->GetChildren();
          while(child2) //for every setup
          {
            wxString name;
            if(child2->GetName() == wxT("ssampling_off")) name = wxT("Supersampling OFF");
            else if(child2->GetName() == wxT("ssampling_on")) name = wxT("Supersampling ON");      
            else
            {
              //read score
              child2->GetNodeContent().ToDouble(&score);   
              complex_score += score;     
              break;
            }
            
            //Get FPS
            tmp = child2->GetChildren();
            tmp->GetNodeContent().ToDouble(&fps);          
            //convert to graphs
            graph_fps = fps;
            if(graph_fps > graph_width) graph_fps=graph_width;                
            //write
            html_core<< wxT(
            "<i>")<<name<<wxT(":</i>"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n");                  
            child2 = child2->GetNext();            
          }      
          html_core<<wxT("</td></tr>\n</table><hr>");
        }         
        child = child->GetNext();  
      }                
    }    
    
    //*********************************************************************************************
    //COMPUTE TESTS
    if(root->GetName() == wxT("compute"))
    {
      child = root->GetChildren();    
      while(child)
      {
        
        //N-BODY SIMULATION
        if (child->GetName() == wxT("nbody"))
        {  
          //get data
          tmp = child->GetChildren();
          tmp->GetNodeContent().ToDouble(&fps); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&fillrate); tmp = tmp->GetNext();
          tmp->GetNodeContent().ToDouble(&score);   
          compute_score += score;   
          //convert to graphs
          graph_fps = fps;
          if(graph_fps > graph_width) graph_fps=graph_width;
          graph = fillrate*5; 
          if(graph > graph_width) graph=graph_width;   
          //write         
          html_core<<wxT( 
            "<table align=\"center\">\n"
            "<tr><th colspan=\"2\">OPENCL COMPUTE TESTS RESULTS</th></tr>\n"  
            "<tr><td class=\"screen\"><img src=\"www/pic/test21.jpg\"></td>\n" 
            "<td><h2>N-Body simulation</h2>\n"
            "<span class=\"grad1\" style=\"width:")<<graph_fps<<wxT("px;\"><b>")<<fps<<wxT("&nbsp;FPS</b></span>\n"
            "<span class=\"grad2\" style=\"width:")<<graph<<wxT("px;\"><b>")<<fillrate<<wxT("&nbsp;GFLOPS</b></span>\n"
            "</td></tr>\n</table><hr>");  
        }         
        child = child->GetNext();  
      }                
    }       
    
    
    root = root->GetNext();
  } 
  
  float total_score = fill_score + frag_score + vert_score + geom_score + complex_score + compute_score;
  
  
  //scores strings, truncate to one point digit
  wxString total_score_str, fill_score_str,frag_score_str,vert_score_str,geom_score_str, complex_score_str, compute_score_str;
  fill_score_str.Printf(wxT("%.1f"),fill_score);
  frag_score_str.Printf(wxT("%.1f"),frag_score);
  geom_score_str.Printf(wxT("%.1f"),geom_score);
  vert_score_str.Printf(wxT("%.1f"),vert_score);
  complex_score_str.Printf(wxT("%.1f"),complex_score);
  compute_score_str.Printf(wxT("%.1f"),compute_score);
  total_score_str.Printf(wxT("%.1f"),total_score);
  
  //set scores to GUI
  textFillScore->SetLabel(fill_score_str);
  textFragScore->SetLabel(frag_score_str);
  textGeomScore->SetLabel(geom_score_str);
  textVertScore->SetLabel(vert_score_str);
  textCplxScore->SetLabel(complex_score_str);
  textCompScore->SetLabel(compute_score_str);
  txtTotScore->SetLabel(total_score_str);
   
 
  html_head<< wxT("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n"
"<html>\n"
" <head>\n"
" <meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-2\">\n"
" <link rel=\"stylesheet\" type=\"text/css\" href=\"www/style.css\">\n"
" <title>gluxMark2 Results</title>\n"
" </head>\n"
" <body>\n"
"  <div align=\"center\">\n"
"  <div class=\"main\">\n"
"   <div class=\"banner\"><a href=\"http://www.stud.fit.vutbr.cz/~xvanek29/gluxmark2/index.php?page=results&action=upload\"></a></div>\n"
"   <div class=\"left\">\n"
//scores
"    <h1>TOTAL SCORE: ")<<total_score_str<<wxT("</h1>\n"
"    <table>\n"
"    <tr><td>FILLRATE SCORE:</td><td>")<<fill_score_str<<wxT("</td></tr>\n"
"    <tr><td>FRAGMENT SHADER SCORE:</td><td>")<<frag_score_str<<wxT("</td></tr>\n"
"    <tr><td>GEOMETRY SHADER SCORE:</td><td>")<<geom_score_str<<wxT("</td></tr>\n"
"    <tr><td>VERTEX SHADER SCORE:</td><td>")<<vert_score_str<<wxT("</td></tr>\n"
"    <tr><td>HDR/COMPLEX SCORE:</td><td>")<<complex_score_str<<wxT("</td></tr>\n"
"    <tr><td>COMPUTE SCORE:</td><td>")<<compute_score_str<<wxT("</td></tr>\n"
"    </table>\n"
"    <h1><a href=\"http://www.stud.fit.vutbr.cz/~xvanek29/gluxmark2/index.php?page=results&action=upload\">&gt;&gt; Upload results &lt;&lt;</a></h1>\n"
//menu
"    <span><a href=\"http://www.stud.fit.vutbr.cz/~xvanek29/gluxmark2/\">Homepage</a></span>\n"
"    <span><a href=\"#info\">System info</a></span>\n"
"    <span><a href=\"#fillrate\">Fillrate tests</a></span>\n"
"    <span><a href=\"#fragment\">Fragment shader tests</a></span>\n"
"    <span><a href=\"#geometry\">Geometry shader tests</a></span>\n"
"    <span><a href=\"#vertex\">Vertex shader tests</a></span>\n"
"    <span><a href=\"#complex\">HDR/Complex tests</a></span>\n"
"    <span><a href=\"#compute\">OpenCL compute tests</a></span>\n"
"    </div>\n");

//end file
html_tail<<wxT(
"   <h2><a href=\"http://www.stud.fit.vutbr.cz/~xvanek29/gluxmark2/index.php?page=results&action=upload\">&gt;&gt; Upload results &lt;&lt;</a></h2>\n"
"   </div>\n"
"   <div class=\"footer\">\n"
"    <p>Author: <a href=\"mailto:xvanek29@stud.fit.vutbr.cz\">Juraj Vanek</a>.\n" 
"      Please contact me in case of any questions or bug-reports.</p>\n"
"  </div>\n"
"  </div>\n"
"  </div>\n"
" </body>\n"
"</html>\n");
             
  //write to HTML page
  wxString html = html_head + html_core + html_tail;
  wxFile file;
  file.Create(wxT("results.html"),wxFile::write);
  if(!file.Write(html))
    return false;
  
  return true;
} 
