/******************************************************************************/
//
//  Program: gluxMark2, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test.h
//  Desc: test class declaration and system info
//
//*****************************************************************************/
#ifndef _TEST_H_
#define _TEST_H_
#include "test_globals.h"

//*********
//ALL TESTS
enum tests
{
  //fillrate
  TEXEL_FILLRATE_SINGLE,
  TEXEL_FILLRATE_MULTI,
  TEXEL_FILLRATE_FLOAT16,
  TEXEL_FILLRATE_FLOAT32,
  //fragment shaders
  FRAG_LIGHTS,
  FRAG_NOISE,
  FRAG_PROCEDURAL,
  FRAG_PARALLAX,
  FRAG_REFRACT,
	//vertex shaders
	VERT_LIGHTS,
	VERT_WAVES,
	VERT_DISPLACE,
	//complex
	COMPLEX_HDR,
	COMPLEX_SSAO,
	COMPLEX_SHADOW,
	COMPLEX_SSAMPLING,
	//OpenCL compute
	COMPUTE_NBODY,
  //geometry shaders - skipping due to problems
  GEOM_TESSELATE,
  GEOM_CUBEMAP,
  GEOM_PARTICLES,
  GEOM_INSTANCING
};

/**
@class Test
@brief contains data about test results and has code to run the tests
****************************************************************************/
class Test
{
  protected:
			  ///pointer to scene we are rendering into
        TScene *s;      
				///current test name
        string testname;
				///average frame rate
        float avg_fps;
				///total test duration
        unsigned duration;
				///test start time
				unsigned start_time; 
        ///test finish time
				unsigned split_time;
        ///total rendered frames
        int total_frames;
        ///test type
        int type;
        ///animation movement,
        float anim;			
        ///actual frames per second
				float fps;				
        ///subtest count (splits)
        int splits;                 
        ///current setup
        int curr_setup;             
        ///partial scenes split sign
        bool splitbit;              
        ///test stop sign
        bool stopbit;     
				///shall we draw scene?
        bool draw_scene;

        ///variable for text
        string msg;

				///partial FPS for every subtest
				float part_FPS[5];
				///final test score
				float test_score;
        ///test importance
				float weight;							
	public:
				//most of these functions will be overloaded
        Test() {}
        Test(int _duration, int type, TScene *ps);
				virtual ~Test() {}
        virtual void SaveResults();
        virtual void Run();
				///@brief Test animation: increment animation variable and perform various animations
				virtual void Animate()
				{ anim += 0.01f; }
				///@brief has test come to an end?
        bool HasEnded()
        { return stopbit; }
				///@brief set initial time after test has been loaded
        void InitTime()
        { start_time = split_time = SDL_GetTicks() + SPLASH; anim = 0.0;  }
				///@brief update FPS count from timer thread
				void UpdateFPS(unsigned _fps)
				{ fps = (float)_fps; }
				///@brief get total rendered frames
				int GetFrames()
				{ return total_frames; }
				///@brief return actual test score
				float GetScore()
				{  return test_score; }
};

//****************
//Save system info
void SystemInfo();

#endif
