/******************************************************************************/
//
//  Program: gluxMark2, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test_compute.h
//  Desc: OpenCL compute tests declaration
//	Code adapted from: ATI Stream SDK, http://developer.amd.com/gpu/ATIStreamSDK/Pages/default.aspx, N-Body simulation
//
//*****************************************************************************/

#ifndef _TEST_COMPUTE_H_
#define _TEST_COMPUTE_H_
#include "test.h"

#ifdef USE_CL
const int BODIES = 4096;
const int MESH_SIZE = 32;

/** 
Test1 - override OpenCL settings, set kernel arguments and run
N-Body simulation
****************************************************************************************************/
class NBodySim : public TCompute
{
	private:
		//N-Body simulation
		cl_uint numBodies;
    cl_float espSqr;                    //softening factor

		cl_float* initPos;                  //initial position
		cl_float* pos;											//current position

		cl_float* initVel;                  //initial velocity
		cl_float* vel;                      //current velocity

    cl_mem updatedPos;                  //buffer: position of particles
    cl_mem updatedVel;                  //buffer: velocity of particles

		//performance info
		cl_ulong start, end;
		float kernel_time, flops, total_flops;

	//overrided functions
	public:
		bool Init();
		bool SetupKernelArgs();
		bool Run();
		bool Destroy();

		float GetPos(int index)
		{ return pos[index]; }
		float GetFLOPS()
		{ return flops; }
		float GetTotalFLOPS()
		{ return total_flops; }
};

/** 
Test2 - override OpenCL settings, set kernel arguments and run
Mesh deformer
****************************************************************************************************
class MeshDef : public TCompute
{
	private:
		cl_uint mesh_size;
		cl_float* vdata;										//current vertex data
		GLint gl_buffer;										//binded OpenGL vertex buffer

    cl_mem upd_vdata;                  //data at memory buffer

		//performance info
		cl_ulong start, end;
		float kernel_time, flops;

	//overrided functions
	public:
		bool Init(int _vertices, GLint _gl_buffer);
		bool SetupKernelArgs();
		bool Run();
		bool Destroy();

		float GetFLOPS()
		{ return flops; }
};
*/

/**
@class TestCompute
@brief Compute test class for graphic render
****************************************************************************/
class TestCompute : public Test
{
  private:
				NBodySim *nbody;
				//MeshDef *mesh_def;
				string name;

				int body,halfbody;
				char objname[32];
  public:
        TestCompute(int duration,int type, TScene *s);
        ~TestCompute();
        void SaveResults();
        void Run();
				void Animate();
};
#endif
#endif
