/******************************************************************************/
//
//  Program: gluxMark2, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test_fill.cpp
//  Desc: fillrate tests implementation
//
//*****************************************************************************/
#include "test_fill.h"

/**
****************************************************************************************************
@brief Fillrate tests initialization
@param _duration test duration
@param _type test type
@param s pointer to scene
****************************************************************************************************/
TestFill::TestFill(int _duration, int _type, TScene *s) : Test(_duration, _type, s)
{
  splits = 1;   //no subscenes in this test series

  //screen resolution
  resx = s->GetResX();
  resy = s->GetResY();

  //setup camera
  s->MoveCameraAbs(0.0f,0.0f,-0.21f);
  s->RotateCameraAbs(90.0,A_X);
  //setup texture surfaces
  s->AddMaterial("fill_mat",white,white,white,64.0f,0.0f,0.1f,NONE);

  glGetIntegerv(GL_MAX_TEXTURE_UNITS, &tex_units);   //texture units
  //number of surfaces
  if(type >= TEXEL_FILLRATE_MULTI)
    surfaces = SURFACES/tex_units;
  else
    surfaces = SURFACES;

  switch(type)
  {
    case TEXEL_FILLRATE_SINGLE:
        testname = "texel_fillrate_single";
				weight = 0.15f;
				s->SetLoadList(surfaces + 2);	//loaded items count (must be determined manually :(
        //add texture
				try
				{
					s->AddTexture("fill_mat","data/tex/single.tga",BASE,MODULATE,1.0,1.0,1.0);
				}
				catch(int err)
				{
					stopbit = true;
					throw ERR;
					return;
				}
        break;

    case TEXEL_FILLRATE_MULTI:
        testname = "texel_fillrate_multi";
				weight = 0.30f;
				s->SetLoadList(surfaces + tex_units + 1);	//loaded items count (must be determined manually :(
				try
				{
					//initialize all texture units with texture
					for(int i = 0; i < tex_units/2; i++)
					{
						s->AddTexture("fill_mat","data/tex/multi.tga",BASE,MODULATE,1.5,1.0,1.0);
						s->AddTexture("fill_mat","data/tex/multi.tga",BASE,ADD,1.5,1.0,1.0);
					}
				}
				catch(int err)
				{
					stopbit = true;
					throw ERR;
					return;
				}
        break;
    case TEXEL_FILLRATE_FLOAT16:
        testname = "texel_fillrate_float16";
				weight = 0.40f;
				s->SetLoadList(surfaces + tex_units + 1);	//loaded items count (must be determined manually :(
				try
				{
					for(int i = 0; i < tex_units/2; i++)
					{
						s->AddTexture("fill_mat","data/tex/multi.tga",BASE,MODULATE,1.5,1.0,1.0);
						s->AddTexture("fill_mat","data/tex/multi.tga",BASE,ADD,1.5,1.0,1.0);
					}
					//HDR render to 16-bit floating point texture
					s->UseHDR(true);
					s->CreateRenderTarget(resx,resy,GL_RGBA16F, GL_FLOAT);

					//set bloom and tonemapping variables
					s->SetUniform("mat_tonemap","exposure",1.0);
					s->SetUniform("mat_tonemap","bloomFactor",1.0);
					s->SetUniform("mat_tonemap","brightMax",1.0);
					s->SetUniform("mat_tonemap","ssao_only",false);
					s->SetUniform("mat_bloom_hdr","THRESHOLD",0.0);
					s->SetUniform("mat_blur_horiz","kernel_size",1);
					s->SetUniform("mat_blur_horiz","blur_radius",0.0);
					s->SetUniform("mat_blur_vert","kernel_size",1);
					s->SetUniform("mat_blur_vert","blur_radius",0.0);
				}
				catch(int err)
				{
					stopbit = true;
					throw ERR;
					return;
				}
        break;
    case TEXEL_FILLRATE_FLOAT32:
        testname = "texel_fillrate_float32";
				weight = 0.15f;
				s->SetLoadList(surfaces + tex_units + 1);	//loaded items count (must be determined manually :(
				try
				{
					for(int i = 0; i < tex_units/2; i++)
					{
						s->AddTexture("fill_mat","data/tex/multi.tga",BASE,MODULATE,1.5,1.0,1.0);
						s->AddTexture("fill_mat","data/tex/multi.tga",BASE,ADD,1.5,1.0,1.0);
					}
					//HDR render to 32-bit floating point texture
					s->UseHDR(true);
					s->CreateRenderTarget(resx,resy,GL_RGBA32F, GL_FLOAT);

					//set bloom and tonemapping variables
					s->SetUniform("mat_tonemap","exposure",1.0);
					s->SetUniform("mat_tonemap","bloomFactor",1.0);
					s->SetUniform("mat_tonemap","brightMax",1.0);
					s->SetUniform("mat_tonemap","ssao_only",false);
					s->SetUniform("mat_bloom_hdr","THRESHOLD",0.0);
					s->SetUniform("mat_blur_horiz","kernel_size",1);
					s->SetUniform("mat_blur_horiz","blur_radius",0.0);
					s->SetUniform("mat_blur_vert","kernel_size",1);
					s->SetUniform("mat_blur_vert","blur_radius",0.0);
				}
				catch(int err)
				{
					stopbit = true;
					throw ERR;
					return;
				}
        break;
    default:
        stopbit = true;
        throw ERR;
        break;
  };

	//surfaces
  for(int i = 0; i < surfaces; i++)
  {
		obj_name = "surface" + num2str(i);
    s->AddObject(obj_name.c_str(),PLANE,1.0,1.0);
    s->MoveObjAbs(obj_name.c_str(),0.0f, -surfaces*0.01f + 0.01f*i,0.0f);
    s->SetMaterial(obj_name.c_str(),"fill_mat");
  }

  if(!s->PostInit())
		stopbit = true;

  //after initialization, set current time
  InitTime();
}

//*****************************************************************************
//Deinitialize tests, free data and save results
TestFill::~TestFill()
{
  s->Destroy();
}

//*****************************************************************************
//Test run
void TestFill::Run()
{
  Test::Run();
  if(!draw_scene) return;   //don't draw scene if has ended or splashscreen is present

  for(int i = 0; i < surfaces; i++)
  {
    obj_name = "surface" + num2str(i);
    s->RotateObj(obj_name.c_str(),sin(anim)*i/200.0f,A_Y);
  }
  s->Redraw();
	msg = num2str((fps*resx*resy)/1000000.0f * SURFACES) + " MTexels/s";
	s->DrawScreenText(msg.c_str(), 90.0, 1.0);
}

//*************************************************************************************************
//Scene animation (every 10ms)
void TestFill::Animate()
{
  Test::Animate();
/*
  for(int i = 0; i < surfaces; i++)
  {
    obj_name = "surface" + num2str(i);
    s->RotateObj(obj_name.c_str(),sin(anim)/100.0*i,A_Y);
  }
*/
}

//*****************************************************************************
//Save results to file
void TestFill::SaveResults()
{
  Test::SaveResults();

  fillrate = avg_fps*resx*resy/1000000.0f*SURFACES;//fillrate calculation

	//normalize score
	switch(type)
	{
		case TEXEL_FILLRATE_SINGLE:
			test_score += fillrate/110;
			break;
		case TEXEL_FILLRATE_MULTI:
			test_score += fillrate/190;
			break;
		case TEXEL_FILLRATE_FLOAT16:
			test_score += fillrate/50;
			break;
		case TEXEL_FILLRATE_FLOAT32:
			test_score += fillrate/12.5f;
			break;
	}
	cout<<"SCORE: "<<test_score<<endl;

  if(type == TEXEL_FILLRATE_SINGLE)
		results<<"<fillrate>\n";

  results<< "<"<<testname<<">\n"<<
                "<fps>"<<avg_fps<<"</fps>\n"<<
                "<fill_rate>"<<fillrate<<"</fill_rate>\n"
								"<score>"<<test_score<<"</score>"
            "</"<<testname<<">\n";

  if(type == TEXEL_FILLRATE_FLOAT32)
		results<<"</fillrate>\n";

}
