/******************************************************************************/
//
//  Program: gluxMark2, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test_frag.cpp
//  Desc: implementation of fragment shaders test series
//
//*****************************************************************************/
#include "test_frag.h"

/**
****************************************************************************************************
@brief Fragment tests initialization
@param _duration test duration
@param _type test type
@param s pointer to scene
****************************************************************************************************/
TestFrag::TestFrag(int _duration, int _type, TScene *s) : Test(_duration, _type, s)
{
  //switch according to test subtype
  if(type == FRAG_LIGHTS)
  {
	  testname = "frag_lights";
		weight = 0.30f;
    splits = 3;
		s->SetLoadList(17);	//loaded items count (must be determined manually :(

		try
		{
			//basic light positions
			lpos1.Set(0.0,10.0,0.0); lpos2.Set(0.0,10.0,0.0); lpos3.Set(0.0,10.0,0.0); lpos4.Set(0.0,10.0,0.0);
			lpos5.Set(0.0,10.0,0.0); lpos6.Set(0.0,10.0,0.0); lpos7.Set(0.0,10.0,0.0); lpos8.Set(0.0,10.0,0.0);

			// setup camera
			s->MoveCamera(0.0,-11.5,-49.0);
			s->RotateCamera(30.0,A_Y);

			//add objects
			s->AddObject("room_bottom",PLANE,50.0,100.0); s->AddObject("room_top",PLANE,50.0,100.0);
			s->AddObject("room_left",PLANE,100.0,30.0);   s->AddObject("room_right",PLANE,100.0,30.0);
			s->AddObject("room_front",PLANE,50.0,30.0);   s->AddObject("room_back",PLANE,50.0,30.0);
			s->AddObject("cube1",CUBE,3.0);
			s->AddObject("cube2",CUBE,6.0);
			s->AddObject("sphere","data/obj/teapot.3ds");
			s->ResizeObj("sphere",0.7f,0.7f,0.7f);
			s->MoveObj("sphere",0.0,-1.5,0.0);

			//transform objects
			s->MoveObj("room_top",0.0,30.0,0.0);     s->MoveObj("room_left",-25.0,15.0,0.0);
			s->MoveObj("room_right",25.0,15.0,0.0);  s->MoveObj("room_front",0.0,15.0,50.0);
			s->MoveObj("room_back",0.0,15.0,-50.0);

			s->MoveObj("sphere",0.0,9.0,0.0); s->MoveObj("cube1",0.0,2.5,0.0); s->MoveObj("cube2",0.0,-3.5,0.0);

			s->RotateObj("room_top",  180.0,A_Z);
			s->RotateObj("room_left",  90.0,A_X);
			s->RotateObj("room_left",  -90.0,A_Z);
			s->RotateObj("room_right", 90.0,A_X);
			s->RotateObj("room_right", 90.0,A_Z);
			s->RotateObj("room_front",-90.0,A_X);
			s->RotateObj("room_back",  90.0,A_X);

			//add 1 starting light
			s->AddLight(0,black,white,white,lpos1,40.0);

			//add materials & textures
			s->AddMaterial("mat_floor",black,white,white);
			s->AddMaterial("mat_wall",black,white,white);
			s->AddMaterial("mat_sphere",black,white,white,64.0);

			s->AddTexture("mat_floor","data/tex/wall.tga",BASE,MODULATE,1.0,2.0,2.0,true,true);
			s->AddTexture("mat_floor","data/tex/normal/wall.tga",BUMP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_wall","data/tex/brick.tga",BASE,MODULATE,1.0,3.0,3.0,true,true);
			s->AddTexture("mat_wall","data/tex/normal/brick.tga",BUMP,MODULATE,1.0,3.0,3.0);
			s->AddTexture("mat_sphere", "data/tex/normal/bump.tga",BUMP,MODULATE,0.2f, 1.0f,1.0f);

			//set materials
			s->SetMaterial("room_left","mat_wall");  s->SetMaterial("room_right","mat_wall");
			s->SetMaterial("room_front","mat_wall"); s->SetMaterial("room_back","mat_wall");
			s->SetMaterial("room_top","mat_floor");  s->SetMaterial("room_bottom","mat_floor");
			s->SetMaterial("sphere","mat_sphere");
			s->SetMaterial("cube1","mat_floor");
			s->SetMaterial("cube2","mat_floor");
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_NOISE)
  {
    testname = "frag_noise";
		weight = 0.20f;
    octaves = 1.0;
    splits = 4;
		s->SetLoadList(5);	//loaded items count (must be determined manually :(

    //camera setup
    s->MoveCamera(0.0,0.0,-10.0);
    s->RotateCamera(90.0,A_X);
		try
		{
			//objects setup
			s->AddObject("sky",PLANE,16.0,10.0);
			s->AddObject("sun",PLANE,16.0,10.0);
			s->MoveObj("sun",0.0f,0.1f,0.0f);

			//materials setup
			s->AddMaterial("noise",white,white,white,64.0,0.0,1.0,NONE);
			s->AddMaterial("mat_sun",white,white,white,64.0,0.0,1.0,NONE);
			s->AddTexture("mat_sun","data/tex/sky_glow.tga");

			s->CustomShader("noise","data/shaders/noise.vert",NULL,"data/shaders/noise.frag");  //custom noise shader

			s->SetMaterial("sky","noise"); s->SetMaterial("sun","mat_sun");

			//set initial uniform values for noise shader
			s->SetUniform("noise","clamp_top",0.7);
			s->SetUniform("noise","clamp_bottom",0.3);
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_PROCEDURAL)
  {
    testname = "frag_procedural";
		weight = 0.10f;
    splits = 1;
		s->SetLoadList(13);	//loaded items count (must be determined manually :(

    s->MoveCamera(0.0,2.0,-10.0);
    s->RotateCamera(15.0,A_X);
    s->RotateCamera(-15.0,A_Y);

		try
		{
			s->AddLight(0,black,silver,white,TVector(3.0,3.0,3.0));

			s->AddObject("teapot","data/obj/teapot.3ds");
			s->ResizeObj("teapot",0.15f,0.15f,0.15f);
			s->MoveObj("teapot",0.0,-1.0,0.0);
			s->AddObject("cube",CUBE,1.0);
			s->AddObject("sphere",SPHERE,1.0,1.0,32,32);
			s->AddObject("sky",SPHERE,20.0,20.0,32,32);
			s->AddObject("cylinder","data/obj/cylinder.3ds");
			s->AddObject("torus",TORUS,0.5,1.25,8,32);
			s->AddObject("cone","data/obj/cone.3ds");

			s->MoveObj("cube",-4.0,0.0,0.0);
			s->MoveObj("sphere",4.0,0.0,0.0);
			s->MoveObj("cylinder",0.0,-4.0,0.0);
			s->MoveObj("torus",-4.0,-4.0,0.0);
			s->MoveObj("cone", 4.0,-4.0,0.0);

			s->AddMaterial("mat_wood",silver,white,white,16);
			s->CustomShader("mat_wood","data/shaders/procedural/wood.vert",NULL,"data/shaders/procedural/wood.frag");
			s->SetMaterial("cube","mat_wood");

			s->AddMaterial("mat_marble",black,white,white,128);
			s->CustomShader("mat_marble","data/shaders/procedural/marble.vert",NULL,"data/shaders/procedural/marble.frag");
			s->SetMaterial("cone","mat_marble");

			s->AddMaterial("mat_bump",black,yellow,red,64);
			s->CustomShader("mat_bump","data/shaders/procedural/bump.vert",NULL,"data/shaders/procedural/bump.frag");
			s->SetMaterial("torus","mat_bump");

			s->AddMaterial("mat_lattice",black,white,white,128);
			s->CustomShader("mat_lattice","data/shaders/procedural/lattice.vert",NULL,"data/shaders/procedural/lattice.frag");
			s->SetMaterial("sphere","mat_lattice");

			s->AddMaterial("mat_brick",black,white,white,128);
			s->CustomShader("mat_brick","data/shaders/procedural/brick.vert",NULL,"data/shaders/procedural/brick.frag");
			s->SetMaterial("cylinder","mat_brick");

			s->AddMaterial("mat_phong",black,white,white,64);
			s->CustomShader("mat_phong","data/shaders/procedural/phong.vert",NULL,"data/shaders/procedural/phong.frag");
			s->SetMaterial("teapot","mat_phong");

			s->AddMaterial("mat_sky");
			s->CustomShader("mat_sky","data/shaders/procedural/noise.vert",NULL,"data/shaders/procedural/noise.frag");
			s->SetMaterial("sky","mat_sky");

			s->SetUniform("mat_sky","clamp_top",0.8);
			s->SetUniform("mat_sky","clamp_bottom",0.2);
			s->SetUniform("mat_sky","octaves",4.0);
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_PARALLAX)
  {
    testname = "frag_parallax";
		weight = 0.20f;
    splits = 3;
		s->SetLoadList(13);	//loaded items count (must be determined manually :(

    s->MoveCamera(0.0,0.0,-10.0);

		try
		{
			s->AddObject("obj",PLANE,17.0,9.0);
			s->RotateObj("obj",90.0,A_X);

			lpos1.Set(0.0,0.0,12.0);
			s->AddLight(0,black,white,white,lpos1);

			s->AddMaterial("mat_basic");
			s->AddMaterial("mat_bump");
			s->AddMaterial("mat_parallax");

			s->AddTexture("mat_basic","data/tex/wall.tga",BASE,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_bump","data/tex/wall.tga",BASE,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_bump","data/tex/normal/wall.tga",BUMP,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_bump","data/tex/normal/tree.tga",BUMP,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/wall.tga",BASE,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/normal/wall.tga",BUMP,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/normal/tree.tga",BUMP,MODULATE,1.0,1.0,1.0);
			s->AddTexture("mat_parallax","data/tex/depth/wall.tga",PARALLAX,MODULATE,0.0015f,1.0f,1.0f);
			s->AddTexture("mat_parallax","data/tex/depth/tree.tga",PARALLAX,MODULATE,0.0015f,1.0f,1.0f);

			s->SetMaterial("obj","mat_basic");
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_REFRACT)
  {
    testname = "frag_refract";
		weight = 0.20f;
    splits = 4;
		s->SetLoadList(13);	//loaded items count (must be determined manually :(

    //setup scene
    s->MoveCamera(0.0,0.0,-12.0);
    s->RotateCamera(30.0,A_Y);
		try
		{
			s->AddObject("obj1",SPHERE,3.0,4.0,48,48);
			s->AddObject("obj2",TORUS,1.0,4.0,32,48);
			s->AddObject("obj3",TORUS,1.0,4.0,32,48);
			s->RotateObj("obj3",90.0f,A_Y);
			s->AddObject("sky",SPHERE,200.0,0.0,32,32);


			s->AddMaterial("mat_ref");
			s->AddMaterial("mat_ref_built");
			s->AddMaterial("mat_ref_chroma");
			s->AddMaterial("mat_ref_chroma_built");
			s->AddMaterial("mat_sky",black,white,white,32.0,0.0,0.0,NONE);

			//cube maps
			const char *cubefiles[] = {"data/tex/cubemaps/posx.tga","data/tex/cubemaps/negx.tga",
																 "data/tex/cubemaps/posy.tga","data/tex/cubemaps/negy.tga",
																 "data/tex/cubemaps/posz.tga","data/tex/cubemaps/negz.tga" };

			s->AddTexture("mat_sky",cubefiles,CUBEMAP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_ref",cubefiles,CUBEMAP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_ref_built",cubefiles,CUBEMAP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_ref_chroma",cubefiles,CUBEMAP,MODULATE,1.0,2.0,2.0);
			s->AddTexture("mat_ref_chroma_built",cubefiles,CUBEMAP,MODULATE,1.0,2.0,2.0);

			s->SetMaterial("sky","mat_sky");
			s->SetMaterial("obj1","mat_ref");
			s->SetMaterial("obj2","mat_ref");
			s->SetMaterial("obj3","mat_ref");

			//load custom shaders with defined parameters
			s->CustomShader("mat_ref","data/shaders/refract.vert",NULL,"data/shaders/refract.frag",NULL,NULL);
			s->CustomShader("mat_ref_built","data/shaders/refract.vert",NULL,"data/shaders/refract.frag",NULL,"#define USE_BUILTIN\n");
			s->CustomShader("mat_ref_chroma","data/shaders/refract.vert",NULL,"data/shaders/refract.frag",NULL,"#define CHROMA\n");
			s->CustomShader("mat_ref_chroma_built","data/shaders/refract.vert",NULL,"data/shaders/refract.frag",NULL,"#define USE_BUILTIN\n#define CHROMA\n");
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }

  //done
  if(!s->PostInit())
  {
		stopbit = true;
		throw ERR;
  }
  InitTime();
}

//**************************************************************************************************
//Test deinitialization, reset scene, delete dynamic data, save results
TestFrag::~TestFrag()
{
  s->Destroy();
}

//**************************************************************************************************
//Test run
void TestFrag::Run()
{
  Test::Run();
  if(!draw_scene) return;   //don't draw scene if has ended or splashscreen is present

  /////////////////////////////////////////////////////////////////////////////////////////////////
  //switch according to test type
  if(type == FRAG_LIGHTS)
  {
    if(splitbit)   //split bit set, save partial results and move to next light setup
    {
      splitbit = false;

      //set lights for actual partial scene
      switch(curr_setup)
      {
        //1 light
        case 1:
            msg = "1 light";
            break;
        //4 light
        case 2:
            msg = "4 lights";
            s->AddLight(1,black,red,red,lpos2);
            s->AddLight(2,black,green,green,lpos3);
            s->AddLight(3,black,blue,blue,lpos4);
            break;
        //8 lights
        case 3:
            msg = "8 lights";
            s->AddLight(4,black,yellow,yellow,lpos5);
            s->AddLight(5,black,magenta,magenta,lpos6);
            s->AddLight(6,black,cyan,cyan,lpos7);
            s->AddLight(7,black,silver,silver,lpos8);
            break;
        default: break;
      }
			s->UpdateLightCount();
    }

    //light movement
    s->MoveLight(0,lpos1);

    if(curr_setup > 1)
    {
      s->MoveLight(1,lpos2);
      s->MoveLight(2,lpos3);
      s->MoveLight(3,lpos4);
    }
    if(curr_setup > 2)
    {
      s->MoveLight(4,lpos5);
      s->MoveLight(5,lpos6);
      s->MoveLight(6,lpos7);
      s->MoveLight(7,lpos8);
    }

    s->RotateCameraAbs(30.0f - anim/3.0f, A_Y);
    s->Redraw();

    //info about light count
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_NOISE)
  {
    if(splitbit)   //move to next noise setup
    {
      splitbit = false;
      //raise and set noise octaves
      octaves *= 2.0;
      s->SetUniform("noise","octaves",octaves);

      switch(curr_setup)
      {
        case 1: msg = "2 octaves"; break;
        case 2: msg = "4 octaves"; break;
        case 3: msg = "8 octaves"; break;
        case 4: msg = "16 octaves"; break;
        default: break;
      }
    }
    //animate noise
    s->SetUniform("noise","anim",anim/5.0);

    s->Redraw();
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_PROCEDURAL)
  {
    s->Redraw();

    //animate noise
    s->SetUniform("mat_bump","anim",anim/5.0);
    s->SetUniform("mat_sky","anim",anim/10.0);
    s->RotateCameraAbs(-anim/2.0f,A_Y);
  }
/////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_PARALLAX)
  {
    if(splitbit)   //move to next setup
    {
      splitbit = false;
      //change material
      switch(curr_setup)
      {
        case 1:
          msg = "Regular per-pixel lighting";
          s->SetMaterial("obj","mat_basic");
          break;
        case 2:
          msg = "Bump mapping";
          s->SetMaterial("obj","mat_bump");
          break;
        case 3:
          msg = "Parallax mapping";
          s->SetMaterial("obj","mat_parallax");
          break;
        default: break;
      }
    }

    //light movement
    s->MoveLight(0,lpos1);

    s->Redraw();
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == FRAG_REFRACT)
  {
    if(splitbit)   //move to next setup
    {
      splitbit = false;
      //change material
      switch(curr_setup)
      {
        case 1:
          msg = "Custom refract function";
          s->SetMaterial("obj1","mat_ref"); s->SetMaterial("obj2","mat_ref"); s->SetMaterial("obj3","mat_ref");
          break;
        case 2:
          msg = "Built-in refract function";
          s->SetMaterial("obj1","mat_ref_built");  s->SetMaterial("obj2","mat_ref_built");  s->SetMaterial("obj3","mat_ref_built");
          break;
        case 3:
          msg = "Custom refract function with chromatic diffusion";
          s->SetMaterial("obj1","mat_ref_chroma"); s->SetMaterial("obj2","mat_ref_chroma"); s->SetMaterial("obj3","mat_ref_chroma");
          break;
        case 4:
          msg = "Built-in refract function with chromatic diffusion";
          s->SetMaterial("obj1","mat_ref_chroma_built"); s->SetMaterial("obj2","mat_ref_chroma_built"); s->SetMaterial("obj3","mat_ref_chroma_built");
					break;
        default: break;
      }
    }
    //rotate camera
    s->RotateCameraAbs(anim*10.0f, A_Y);
    s->Redraw();
    s->DrawScreenText(msg.c_str(),50,95,1.2f);
  }
}

//*************************************************************************************************
//Scene animation (every 10ms)
void TestFrag::Animate()
{
  Test::Animate();
	switch(type)
	{
	case FRAG_LIGHTS:
		//s->RotateCameraAbs(30.0 - anim/3.0, A_Y);
    //light movement
    lpos1.x = 10.0f*cos(anim);
    lpos1.z = 45.0f*sin(anim);
    //s->MoveLight(0,lpos1);

    if(curr_setup > 1)
    {
      lpos2.y = 10.0f*cos(1.5f*anim) + 10.0f;
      lpos2.z = 45.0f*sin(1.5f*anim);
      //s->MoveLight(1,lpos2);

      lpos3.x = 10.0f*sin(anim) - 10.0f;
      lpos3.y = 10.0f*cos(0.5f*anim) + 10.0f;
      lpos3.z = 45.0f*cos(0.5f*anim);
      //s->MoveLight(2,lpos3);

      lpos4.x = 10.0f*sin(0.4f*anim) + 10.0f;
      lpos4.y = 10.0f*sin(0.7f*anim) + 10.0f;
      lpos4.z = 45.0f*sin(0.7f*anim);
      //s->MoveLight(3,lpos4);
    }
    if(curr_setup > 2)
    {
      lpos5.x = 20.0f*cos(1.3f*anim);
      lpos5.z = 20.0f*sin(1.3f*anim);
      //s->MoveLight(4,lpos5);

      lpos6.y = 10.0f*cos(1.8f*anim) + 10.0f;
      lpos6.z = 10.0f*sin(1.8f*anim);
      //s->MoveLight(5,lpos6);

      lpos7.x = 10.0f*sin(1.6f*anim) - 10.0f;
      lpos7.y = 10.0f*cos(1.6f*anim) + 10.0f;
      lpos7.z = 10.0f*cos(1.6f*anim);
      //s->MoveLight(6,lpos7);

      lpos8.x = 10.0f*sin(0.9f*anim) + 10.0f;
      lpos8.y = 10.0f*sin(0.8f*anim) + 10.0f;
      lpos8.z = 10.0f*sin(0.8f*anim);
      //s->MoveLight(7,lpos8);
    }
	case FRAG_PROCEDURAL:
		//s->RotateCameraAbs(-anim/2.0,A_Y);
		break;
	case FRAG_PARALLAX:
		//light movement
    lpos1.x = 10.0f*cos(anim);
    lpos1.y = 10.0f*sin(anim);
    //s->MoveLight(0,lpos1);
		break;
	case FRAG_REFRACT:
		//s->RotateCameraAbs(anim*10.0, A_Y);
		break;
	default:
		break;
	}
}

//**************************************************************************************************
//Save results to XML file
void TestFrag::SaveResults()
{
  Test::SaveResults();

  //switch according to test type
  if(type == FRAG_LIGHTS)
  {
    //test name
    switch(curr_setup)
    {
      case 1:
          results<<"<fragment>\n"
                  "<"<<testname<<">\n"
                  "<light1>\n<fps>"<<avg_fps<<"</fps>\n</light1>\n";
					test_score += avg_fps/7.5f;
          break;
      case 2:
          results<<"<light4>\n<fps>"<<avg_fps<<"</fps>\n</light4>\n";
					test_score += avg_fps/4;
          break;
      case 3:
					test_score += avg_fps/2;
          results<<"<light8>\n<fps>"<<avg_fps<<"</fps>\n</light8>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == FRAG_NOISE)
  {
    //write performance for every noise octave
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<oct2>\n<fps>"<<avg_fps<<"</fps>\n</oct2>\n";
					test_score += avg_fps/8;
          break;
      case 2:
          results<<"<oct4>\n<fps>"<<avg_fps<<"</fps>\n</oct4>\n";
					test_score += avg_fps/9;
          break;
      case 3:
          results<<"<oct8>\n<fps>"<<avg_fps<<"</fps>\n</oct8>\n";
					test_score += avg_fps/3;
          break;
      case 4:
					test_score += avg_fps/2;
          results<<"<oct16>\n<fps>"<<avg_fps<<"</fps>\n</oct16>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == FRAG_PARALLAX)
  {
    //write performance for every refraction setup
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<perpixel>\n<fps>"<<avg_fps<<"</fps>\n</perpixel>\n";
					test_score += avg_fps/50;
          break;
      case 2:
          results<<"<bump>\n<fps>"<<avg_fps<<"</fps>\n</bump>\n";
					test_score += avg_fps/20;
          break;
      case 3:
					test_score += avg_fps/6;
          results<<"<parallax>\n<fps>"<<avg_fps<<"</fps>\n</parallax>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == FRAG_REFRACT)
  {
    //write performance for every refraction setup
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<ref_custom>\n<fps>"<<avg_fps<<"</fps>\n</ref_custom>\n";
					test_score += avg_fps/16;
          break;
      case 2:
          results<<"<ref_builtin>\n<fps>"<<avg_fps<<"</fps>\n</ref_builtin>\n";
					test_score += avg_fps/16;
          break;
      case 3:
          results<<"<ref_custom_chroma>\n<fps>"<<avg_fps<<"</fps>\n</ref_custom_chroma>\n";
					test_score += avg_fps/12;
          break;
      case 4:
					test_score += avg_fps/12;
          results<<"<ref_builtin_chroma>\n<fps>"<<avg_fps<<"</fps>\n</ref_builtin_chroma>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n"
              "</fragment>\n";    //end of fragment shader tests
          break;
      default: break;
    }
  }
  //else write only test name and FPS
  else
  {
		test_score = avg_fps/2;
    results<<"<"<<testname<<">\n"
           "<fps>"<<avg_fps<<"</fps>\n"
					 "<score>"<<test_score<<"</score>"
           "</"<<testname<<">\n";
  }
}
