/******************************************************************************/
//
//  Program: gluxMark2, DIP
//  Author: Juraj Vanek, xvanek29@stud.fit.vutbr.cz
//  File: test_geom.cpp
//  Desc: implementation of geometry shaders test series
//
//*****************************************************************************/
#include "test_geom.h"

/**
****************************************************************************************************
@brief Geometry shader tests initialization
@param _duration test duration
@param _type test type
@param s pointer to scene
****************************************************************************************************/
TestGeom::TestGeom(int _duration, int _type, TScene *s) : Test(_duration, _type, s)
{
  //check for necessary extensions
  if(!GLEW_EXT_geometry_shader4 && !GLEW_EXT_gpu_shader4)
  {
    cout<<"No HW support for geometry shaders - skipping test\n";
    stopbit = true;
    return;
  }

  //switch according to test subtype
  if(type == GEOM_TESSELATE)
  {
	  testname = "geom_tesselate";
		weight = 0.15f;
    splits = 4;
		s->SetLoadList(8);	//loaded items count (must be determined manually :(

    lpos.Set(500.0,100.0,-100.0);

		try
		{
			s->AddObject("obj",STRIP_PLANE,100.0,300.0,25,125);
			s->AddObject("obj2",STRIP_PLANE,100.0,300.0,25,125);
			s->AddObject("sky", PLANE, 770.0,200.0);
			s->MoveObj("obj2",0.0,0.0,300.0);
			s->RotateObj("sky",90.0,A_X);

			s->AddLight(0,silver,white,white,lpos);

			s->AddMaterial("mat_tess",lgrey);
			s->AddTexture("mat_tess", "data/tex/snow2.tga",BASE);
			s->AddTexture("mat_tess", "data/tex/depth/terrain.tga",BUMP,MODULATE,1.0,1.0,1.0,false);
			//s->AddTexture("mat_tess", "data/tex/normal/terrain.tga",BUMP);

			s->AddMaterial("mat_sky",white,white,white,0.0,0.0,0.0,NONE);
			s->AddTexture("mat_sky", "data/tex/sky_glow.tga",BASE);

			s->CustomShader("mat_tess","data/shaders/geometry/subd.vert","data/shaders/geometry/subd.geom","data/shaders/geometry/subd.frag");

			s->SetMaterial("obj","mat_tess");  s->SetMaterial("obj2","mat_tess");
			s->SetMaterial("sky","mat_sky");
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == GEOM_CUBEMAP)
  {
    testname = "geom_cubemap";
		weight = 0.15f;
    splits = 2;
    OBJECT_COUNT = 7;
		s->SetLoadList(11 + OBJECT_COUNT*OBJECT_COUNT);	//loaded items count (must be determined manually :(

    lpos.Set(2.0,5.0,3.0);
    s->MoveCamera(0.0,0.0,-5.0);
    s->RotateCamera(15.0,A_Y);
    s->RotateCamera(15.0,A_X);

		try
		{
			//add objects
			s->AddObject("ref_sphere",SPHERE,1.0,1.0,32,32);
			s->AddObject("ref_sphere2",SPHERE,0.5,0.5,32,32);
			s->AddObject("sky",SPHERE,100.0,0.0,32,32);//"data/obj/gula.3ds");

			//add light(s)
			s->AddLight(0,silver,TVector(1.0f,0.9f,0.8f),white,lpos);

			//create materials
			s->AddMaterial("mat_sphere",dgrey,white,white,32.0,1.0,0.0, GOURAUD);
			s->AddMaterial("mat_sphere2",dgrey,dgrey,white,32.0,1.0,0.0, GOURAUD);
			s->AddMaterial("mat_fighter",black,white,white,64.0, 0.0, 0.0, GOURAUD);
			s->AddMaterial("mat_sky",white,white,white,1.0,0.0,0.0,NONE);

			//add textures
			s->AddTexture("mat_sky", "data/tex/stars.tga",BASE, MODULATE, 1.0, 4.0,4.0);
			s->AddTexture("mat_fighter", "data/tex/fighter.tga",BASE);
			s->AddTexture("mat_sphere", "data/tex/metal.tga",BASE, MODULATE, 0.5, 2.0, 2.0);
			s->AddTexture("mat_sphere2", "data/tex/metal.tga",BASE, MODULATE, 0.5, 2.0, 2.0);

			//setup materials
			s->SetMaterial("ref_sphere", "mat_sphere");
			s->SetMaterial("ref_sphere2", "mat_sphere2");
			s->SetMaterial("sky", "mat_sky");

			//many planes
			float rnd1, rnd2, rnd3;
			s->AddObject("fighter0","data/obj/fighter.3ds");
			s->SetMaterial("fighter0", "mat_fighter");
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd1 = rand()%100 / 100.0f - 2.0f;
					rnd2 = rand()%100 / 100.0f - 2.0f;
					rnd3 = rand()%10 / 10.0f;
					obj_name = "fighter" + num2str(i*OBJECT_COUNT + j + 1);
					s->AddObjectInstance("fighter0",obj_name.c_str());
					s->MoveObj(obj_name.c_str(), 40.0f, (2.0f * i + rnd1) - (2.0f * OBJECT_COUNT/2), (5.0f * j + rnd2) - (5.0f * OBJECT_COUNT/2) );
					speeds.push_back(rnd3);
				}
			}
			s->SetReflectionSize("ref_sphere",512);
			s->SetReflectionSize("ref_sphere2",512);
			s->UseDynReflections(false);      //create dynamic cubemaps
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}

    //initial setup
    msg1 = "Using multi-pass cubemapping";
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == GEOM_PARTICLES)
  {
    testname = "geom_particles";
		weight = 0.50;
    splits = 4;
    OBJECT_COUNT = 20;
		s->SetLoadList(12 + OBJECT_COUNT*OBJECT_COUNT);	//loaded items count (must be determined manually :(

    s->MoveCamera(-4.0,0.0,-15.0);
    s->RotateCamera(35.0,A_X);

		try
		{
			lpos.Set(0.0,10.0,10.0);
			s->AddLight(0,black,dgrey,white,lpos);

			//wet plane
			s->AddObject("wet_plane",PLANE,50.0,50.0);

			s->AddMaterial("mat_rain1",black,dgrey,white,64.0,0.0,1.0);
			s->AddTexture("mat_rain1","data/tex/alpha/raindrop.tga");
			s->AddMaterial("mat_rain2",black,dgrey,white,64.0,0.0,1.0);
			s->AddTexture("mat_rain2","data/tex/alpha/raindrop.tga");
			s->AddMaterial("mat_rain3",black,dgrey,white,64.0,0.0,1.0);
			s->AddTexture("mat_rain3","data/tex/alpha/raindrop.tga");
			s->AddMaterial("mat_rain4",black,dgrey,white,64.0,0.0,1.0);
			s->AddTexture("mat_rain4","data/tex/alpha/raindrop.tga");

			s->AddMaterial("mat_wet",black,dgrey,white,8.0);
			s->AddTexture("mat_wet","data/tex/wall.tga");
			s->AddTexture("mat_wet","data/tex/normal/bump.tga",BUMP);
			s->AddTexture("mat_wet","data/tex/normal/raindrops.tga",BUMP);
			s->SetMaterial("wet_plane","mat_wet");

			//position rain emitters
			float rnd;
			s->AddObject("emitter0",PLANE,10.0,10.0);
			s->SetMaterial("emitter0", "mat_rain1");
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd = (rand()%10)/5.0f - 1.0f;
					obj_name = "emitter" + num2str(i*OBJECT_COUNT + j + 1);
					s->AddObjectInstance("emitter0",obj_name.c_str());
					s->MoveObj(obj_name.c_str(), 1.5f*j + rnd - OBJECT_COUNT/2.0f, 10.0f - 10.0f*rnd, 1.5f*i - OBJECT_COUNT/2.0f);
				}
			}
			//rain shaders
			s->CustomShader("mat_rain1","data/shaders/geometry/rain.vert","data/shaders/geometry/rain.geom","data/shaders/geometry/rain.frag",NULL,NULL,
				"#define PARTICLES 2\nlayout(max_vertices=16) out;");
			s->CustomShader("mat_rain2","data/shaders/geometry/rain.vert","data/shaders/geometry/rain.geom","data/shaders/geometry/rain.frag",NULL,NULL,
				"#define PARTICLES 4\nlayout(max_vertices=64) out;");
			s->CustomShader("mat_rain3","data/shaders/geometry/rain.vert","data/shaders/geometry/rain.geom","data/shaders/geometry/rain.frag",NULL,NULL,
				"#define PARTICLES 6\nlayout(max_vertices=144) out;");
			s->CustomShader("mat_rain4","data/shaders/geometry/rain.vert","data/shaders/geometry/rain.geom","data/shaders/geometry/rain.frag",NULL,NULL,
				"#define PARTICLES 8\nlayout(max_vertices=255) out;");

			s->CustomShader("mat_wet","data/shaders/wet.vert",NULL,"data/shaders/wet.frag");

			//starting setup
			s->SetUniform("mat_wet","rain_speed",20.0);
			msg1 = "1600 particles";
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == GEOM_INSTANCING)
  {
    testname = "geom_instancing";
		weight = 0.20f;
    splits = 2;
    OBJECT_COUNT = 12;
		s->SetLoadList(14 + OBJECT_COUNT*OBJECT_COUNT);	//loaded items count (must be determined manually :(

    s->MoveCamera(0.0,-2.0,0.0);

		try
		{
			lpos.Set(0.0,15.0,50.0);
			s->AddLight(0,grey,white,white,lpos);

			//materials
			s->AddMaterial("mat_rock",dgrey,grey,white,64.0,0.0,0.0,GOURAUD);
			s->AddMaterial("mat_rock_inst",dgrey,grey,white);
			s->AddTexture("mat_rock","data/tex/asteroid.tga",BASE,MODULATE,1.0,4.0,4.0);
			s->AddTexture("mat_rock_inst","data/tex/asteroid.tga");

			//objects to be drawn as copies coming from GS
			s->AddObject("rockA","data/obj/rock.3ds");
			s->AddObject("rockB","data/obj/rock.3ds");
			s->AddObject("rockC","data/obj/rock.3ds");
			s->AddObject("rockD","data/obj/rock.3ds");
			s->SetMaterial("rockA","mat_rock_inst");
			s->SetMaterial("rockB","mat_rock_inst");
			s->SetMaterial("rockC","mat_rock_inst");
			s->SetMaterial("rockD","mat_rock_inst");
			s->MoveObj("rockA", 0.0f, -3.0f, 0.0f);
			s->MoveObj("rockB", 5.9f, -3.2f, 5.1f);
			s->MoveObj("rockC", 0.5f, -3.2f, 5.2f);
			s->MoveObj("rockD", 5.1f, -3.0f, 0.4f);

			//sky (space :)
			s->AddObject("sky",SPHERE,100.0,0.0,16,16);
			s->AddMaterial("mat_sky",white,white,white,64.0,0.0,0.0,NONE);
			s->AddTexture("mat_sky","data/tex/stars.tga",BASE,MODULATE,1.0,4.0,4.0);
			s->SetMaterial("sky","mat_sky");

			//sun
			s->AddObject("star",PLANE,10.0,10.0);
			s->AddMaterial("mat_star",white,white,white,64.0,0.0,0.0,NONE);
			s->AddTexture("mat_star","data/tex/star.tga",BASE,MODULATE,2.0);
			s->SetMaterial("star","mat_star");
			s->MoveObj("star",lpos.x, lpos.y, lpos.z);
			s->RotateObj("star",90.0,A_X);

			//shader performing geometry instancing
			s->CustomShader("mat_rock_inst","data/shaders/geometry/instanced.vert","data/shaders/geometry/instanced.geom","data/shaders/geometry/instanced.frag");

			//initialize all objects with random position
			float rnd;
			s->AddObject("rock0","data/obj/rock.3ds");
			s->SetMaterial("rock0", "mat_rock");
			for(int i=0; i<OBJECT_COUNT; i++)
			{
				for(int j=0; j<OBJECT_COUNT; j++)
				{
					rnd = rand()%100 / 25.0f - 2.0f;
					obj_name = "rock" + num2str(i*OBJECT_COUNT + j + 1);
					s->AddObjectInstance("rock0",obj_name.c_str());
					s->MoveObj(obj_name.c_str(), j*4.0f + rnd - 2.0f*OBJECT_COUNT, rnd/4.0f, i*4.0f - rnd -2.0f* OBJECT_COUNT);
				}
			}
			//for start, turn off GI
			for(int i=0; i<OBJECT_COUNT*OBJECT_COUNT; i++)
			{
				obj_name = "rock" + num2str(i);
				s->DrawObject(obj_name.c_str(),true);
			}
			s->DrawObject("rockA",false);
			s->DrawObject("rockB",false);
			s->DrawObject("rockC",false);
			s->DrawObject("rockD",false);
			msg1 = "Geometry instancing OFF - drawing 144 objects and 0 instances";
		}
		catch(int err)
		{
			stopbit = true;
			throw ERR;
			return;
		}
  }

  //done
  if(!s->PostInit())
  {
		stopbit = true;
		throw ERR;
  }
  InitTime();
}

//**************************************************************************************************
//Test deinitialization, reset scene, delete dynamic data, save results
TestGeom::~TestGeom()
{
  //restore viewport
  int resx = s->GetResX();
  int resy = s->GetResY();
  s->Resize(resx,resy);

  s->Destroy();
}

//**************************************************************************************************
//Test run
void TestGeom::Run()
{
  Test::Run();
  if(!draw_scene)
		return;   //don't draw scene if has ended or splashscreen is present

  /////////////////////////////////////////////////////////////////////////////////////////////////
  //switch according to test type
  if(type == GEOM_TESSELATE)
  {
    if(splitbit)   //increase tesselation level
    {
      splitbit = false;

      //set tesselation level and output vertices
      switch(curr_setup)
      {
				case 1: s->SetUniform("mat_tess","TESS_LEVEL",0); break;
        case 2: s->SetUniform("mat_tess","TESS_LEVEL",1); break;
        case 3: s->SetUniform("mat_tess","TESS_LEVEL",2); break;
        case 4: s->SetUniform("mat_tess","TESS_LEVEL",3); break;
        default: break;
      }

      msg = "Tesselation level " + num2str(curr_setup);
    }

    s->MoveCameraAbs(-12.0f,-3.0f,-400.0f + anim);
    s->MoveObjAbs("sky",0.0f,75.0f, -anim);

    //scene resolution
    int resx = s->GetResX();
    int resy = s->GetResY();

    //1.viewport = normal
		s->DrawObject("sky",true);
    glViewport(0,0,resx/2,resy);        //new viewport settings
    glMatrixMode(GL_PROJECTION);      //projection matrix
    glLoadIdentity();
    //perspective settings
    gluPerspective(45.0,(GLfloat)resx/resy,0.1,1000.0);
    glMatrixMode(GL_MODELVIEW);       //new modelview matrix
    s->Redraw();

    //2.viewport = wireframe
		s->DrawObject("sky",false);
    glViewport(resx/2,0,resx/2,resy);        //new viewport settings
    glMatrixMode(GL_PROJECTION);      //projection matrix
    glLoadIdentity();
    //perspective settings
    gluPerspective(45.0,(GLfloat)resx/resy,0.1,1000.0);
    glMatrixMode(GL_MODELVIEW);       //new modelview matrix
    //draw in wireframe
    glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
    s->Redraw(false);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

		//restore viewport
		glViewport(0,0,resx,resy);
		s->DrawScreenText(msg.c_str(),50.0,95.0f,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == GEOM_CUBEMAP)
  {
    if(splitbit)   //turn on geometry cubemapping
    {
      splitbit = false;

      //turn on geometry cubemapping
      if(curr_setup > 1)
      {
        try
        {
				  s->UseDynReflections(true);
        }
        catch(int err)
        {
          stopbit = true;
          return;
        }
        msg1 = "Using single-pass GS cubemapping";
      }
    }

    //spaceships movement
    for(int i=0; i<OBJECT_COUNT*OBJECT_COUNT; i++)
    {
      obj_name = "fighter" + num2str(i);
      s->MoveObj(obj_name.c_str(), -speeds[i]/fps*30.0f,0.0f,0.0f);
      if(s->GetObjPosition(obj_name.c_str()).x < -40.0)
        s->MoveObj(obj_name.c_str(), 80.0, 0.0, 0.0);
    }

    s->MoveObjAbs("ref_sphere2",sin(anim/5.0f + 4.0f) * 2.0f, 0.0f, cos(anim/5.0f + 4.0f) * 2.0f);
    s->Redraw();

    s->DrawScreenText(msg1.c_str(),50.0f,95.0f,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == GEOM_PARTICLES)
  {
    if(splitbit)   //split bit set, save partial results and increase particle count
    {
      splitbit = false;
			int i;

      //set particle count
      switch(curr_setup)
      {
        case 2:
					for(i=0; i<OBJECT_COUNT*OBJECT_COUNT; i++)
					{
						obj_name = "emitter" + num2str(i);
						s->SetMaterial(obj_name.c_str(), "mat_rain2");
					}
          s->SetUniform("mat_wet","rain_speed",25.0);
          msg1 = "6400 particles";
          break;
        case 3:
 					for(i=0; i<OBJECT_COUNT*OBJECT_COUNT; i++)
					{
						obj_name = "emitter" + num2str(i);
						s->SetMaterial(obj_name.c_str(), "mat_rain3");
					}
          s->SetUniform("mat_wet","rain_speed",30.0);
          msg1 = "14400 particles";
          break;
        case 4:
					for(i=0; i<OBJECT_COUNT*OBJECT_COUNT; i++)
					{
						obj_name = "emitter" + num2str(i);
						s->SetMaterial(obj_name.c_str(), "mat_rain4");
					}
          s->SetUniform("mat_wet","rain_speed",40.0);
          msg1 = "25600 particles";
          break;
        default: break;
      }
    }
    s->SetUniform("mat_rain1","time",anim/2.0);
		s->SetUniform("mat_rain2","time",anim/2.0);
		s->SetUniform("mat_rain3","time",anim/2.0);
		s->SetUniform("mat_rain4","time",anim/2.0);

    s->SetUniform("mat_wet","time",anim/2.0);
    s->Redraw();
    s->DrawScreenText(msg1.c_str(),50.0f,95.0f,1.2f);
  }
  /////////////////////////////////////////////////////////////////////////////////////////////////
  else if(type == GEOM_INSTANCING)
  {
    if(splitbit)   //split bit set, save partial results and turn on GI level
    {
      splitbit = false;
      //turn on GI
      if(curr_setup > 1)
      {
        for(int i=0; i<OBJECT_COUNT*OBJECT_COUNT; i++)
        {
					obj_name = "rock" + num2str(i);
          s->DrawObject(obj_name.c_str(),false);
        }
        s->DrawObject("rockA",true);
        s->DrawObject("rockB",true);
        s->DrawObject("rockC",true);
				s->DrawObject("rockD",true);
        msg1 = "Geometry instancing ON - drawing 4 objects and 140 instances";
      }

    }

    s->RotateCameraAbs(180.0f + anim*18.0f,A_Y);
    s->Redraw();
    s->DrawScreenText(msg1.c_str(),50.0f,95.0f,1.2f);
  }
}

//*************************************************************************************************
//Scene animation (every 10ms)
void TestGeom::Animate()
{
  Test::Animate();
	/*
	switch(type)
	{
	case:
		break;
	}
	*/
}


//**************************************************************************************************
//Save results to XML file
void TestGeom::SaveResults()
{
  Test::SaveResults();

	//there's bug in ATI drivers (no tesselation occurs), try to detect it by compare framerates (should be dropping)
	static float ati_bug[4];

  //switch according to test type
  if(type == GEOM_TESSELATE)
  {
		int VERTICES = 3125;
    //node begin
    switch(curr_setup)
    {
      case 1:
          results<<"<geometry>\n"<<
                  "<"<<testname<<">\n"<<
                  "<level0>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<(9 * VERTICES * avg_fps)/1000000.0<<"</vert_rate>\n</level0>\n";
					test_score += avg_fps/5;
					ati_bug[0] = avg_fps;
          break;
      case 2:
          results<<"<level1>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<(27 * VERTICES * avg_fps)/1000000.0<<"</vert_rate>\n</level1>\n";
					test_score += avg_fps/3;
					ati_bug[1] = avg_fps;
          break;
      case 3:
          results<<"<level2>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<(81 * VERTICES * avg_fps)/1000000.0<<"</vert_rate>\n</level2>\n";
					test_score += avg_fps/2;
					ati_bug[2] = avg_fps;
          break;
      case 4:
					test_score += avg_fps;
					//detect ATI driver bug
					if( !(ati_bug[0] > ati_bug[1] && ati_bug[1] > ati_bug[2] && ati_bug[2] > avg_fps) )
						test_score = 0;

					cout<<"SCORE: "<<test_score<<endl;
          results<<"<level3>\n<fps>"<<avg_fps<<"</fps>\n<vert_rate>"<<(243 * VERTICES * avg_fps)/1000000.0<<"</vert_rate>\n</level3>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == GEOM_CUBEMAP)
  {
    //GS cubemapping on/off
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<GS_off>\n<fps>"<<avg_fps<<"</fps>\n</GS_off>\n";
          break;
      case 2:
					//final weight
					test_score = 0;//@todo: avg_fps;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<GS_on>\n<fps>"<<avg_fps<<"</fps>\n</GS_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == GEOM_PARTICLES)
  {
    //write performance for every particle count setup
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<part_1>\n<fps>"<<avg_fps<<"</fps>\n<part_rate>"<<1600 * avg_fps<<"</part_rate>\n</part_1>\n";
					test_score += avg_fps/10;
          break;
      case 2:
          results<<"<part_2>\n<fps>"<<avg_fps<<"</fps>\n<part_rate>"<<6400 * avg_fps<<"</part_rate>\n</part_2>\n";
					test_score += avg_fps/2.5f;
          break;
      case 3:
          results<<"<part_3>\n<fps>"<<avg_fps<<"</fps>\n<part_rate>"<<14400 * avg_fps<<"</part_rate>\n</part_3>\n";
					test_score += avg_fps*2.5f;
          break;
      case 4:
					test_score += avg_fps*4;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<part_4>\n<fps>"<<avg_fps<<"</fps>\n<part_rate>"<<25600 * avg_fps<<"</part_rate>\n</part_4>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n";
          break;
      default: break;
    }
  }
  else if(type == GEOM_INSTANCING)
  {
    //write performance for GI on/off
    switch(curr_setup)
    {
      case 1:
          results<<"<"<<testname<<">\n"<<
                  "<inst_off>\n<fps>"<<avg_fps<<"</fps>\n</inst_off>\n";
          break;
      case 2:
					//final weight
					test_score = avg_fps/3;
					cout<<"SCORE: "<<test_score<<endl;
          results<<"<inst_on>\n<fps>"<<avg_fps<<"</fps>\n</inst_on>\n"
							"<score>"<<test_score<<"</score>"
              "</"<<testname<<">\n"
              "</geometry>\n";
          break;
      default: break;
    }
  }
}
